@@ -1,17 +1,18 @@
-from html import escape
+from __future__ import annotations
+
+import warnings
 from copy import deepcopy
 from datetime import datetime
-
-from pystac.errors import STACTypeError
-from pystac.html.jinja_env import get_jinja_env
+from html import escape
 from typing import (
+    TYPE_CHECKING,
     Any,
     Dict,
     Iterable,
     List,
     Optional,
-    TYPE_CHECKING,
     Tuple,
+    Type,
     TypeVar,
     Union,
     cast,
@@ -20,25 +21,27 @@ from typing import (
 from dateutil import tz
 
 import pystac
-from pystac import STACObjectType, CatalogType
+from pystac import CatalogType, STACObjectType
 from pystac.asset import Asset
 from pystac.catalog import Catalog
+from pystac.errors import DeprecatedWarning, ExtensionNotImplemented, STACTypeError
+from pystac.html.jinja_env import get_jinja_env
 from pystac.layout import HrefLayoutStrategy
 from pystac.link import Link
 from pystac.provider import Provider
-from pystac.utils import datetime_to_str, str_to_datetime
 from pystac.serialization import (
-    identify_stac_object_type,
     identify_stac_object,
+    identify_stac_object_type,
     migrate_to_latest,
 )
 from pystac.summaries import Summaries
+from pystac.utils import datetime_to_str, str_to_datetime
 
 if TYPE_CHECKING:
-    from pystac.item import Item as Item_Type
-    from pystac.provider import Provider as Provider_Type
+    from pystac.item import Item
+
+C = TypeVar("C", bound="Collection")
 
-T = TypeVar("T")
 TemporalIntervals = Union[List[List[datetime]], List[List[Optional[datetime]]]]
 TemporalIntervalsLike = Union[
     TemporalIntervals, List[datetime], List[Optional[datetime]]
@@ -91,18 +94,19 @@ class SpatialExtent:
         d = {"bbox": self.bboxes, **self.extra_fields}
         return d
 
-    def clone(self) -> "SpatialExtent":
+    def clone(self) -> SpatialExtent:
         """Clones this object.
 
         Returns:
             SpatialExtent: The clone of this object.
         """
-        return SpatialExtent(
+        cls = self.__class__
+        return cls(
             bboxes=deepcopy(self.bboxes), extra_fields=deepcopy(self.extra_fields)
         )
 
     @staticmethod
-    def from_dict(d: Dict[str, Any]) -> "SpatialExtent":
+    def from_dict(d: Dict[str, Any]) -> SpatialExtent:
         """Constructs a SpatialExtent from a dict.
 
         Returns:
@@ -115,7 +119,7 @@ class SpatialExtent:
     @staticmethod
     def from_coordinates(
         coordinates: List[Any], extra_fields: Optional[Dict[str, Any]] = None
-    ) -> "SpatialExtent":
+    ) -> SpatialExtent:
         """Constructs a SpatialExtent from a set of coordinates.
 
         This method will only produce a single bbox that covers all points
@@ -229,18 +233,19 @@ class TemporalExtent:
         d = {"interval": encoded_intervals, **self.extra_fields}
         return d
 
-    def clone(self) -> "TemporalExtent":
+    def clone(self) -> TemporalExtent:
         """Clones this object.
 
         Returns:
             TemporalExtent: The clone of this object.
         """
-        return TemporalExtent(
+        cls = self.__class__
+        return cls(
             intervals=deepcopy(self.intervals), extra_fields=deepcopy(self.extra_fields)
         )
 
     @staticmethod
-    def from_dict(d: Dict[str, Any]) -> "TemporalExtent":
+    def from_dict(d: Dict[str, Any]) -> TemporalExtent:
         """Constructs an TemporalExtent from a dict.
 
         Returns:
@@ -263,7 +268,7 @@ class TemporalExtent:
         )
 
     @staticmethod
-    def from_now() -> "TemporalExtent":
+    def from_now() -> TemporalExtent:
         """Constructs an TemporalExtent with a single open interval that has
         the start time as the current time.
 
@@ -319,20 +324,21 @@ class Extent:
 
         return d
 
-    def clone(self) -> "Extent":
+    def clone(self) -> Extent:
         """Clones this object.
 
         Returns:
             Extent: The clone of this extent.
         """
-        return Extent(
+        cls = self.__class__
+        return cls(
             spatial=self.spatial.clone(),
             temporal=self.temporal.clone(),
             extra_fields=deepcopy(self.extra_fields),
         )
 
     @staticmethod
-    def from_dict(d: Dict[str, Any]) -> "Extent":
+    def from_dict(d: Dict[str, Any]) -> Extent:
         """Constructs an Extent from a dict.
 
         Returns:
@@ -348,8 +354,8 @@ class Extent:
 
     @staticmethod
     def from_items(
-        items: Iterable["Item_Type"], extra_fields: Optional[Dict[str, Any]] = None
-    ) -> "Extent":
+        items: Iterable[Item], extra_fields: Optional[Dict[str, Any]] = None
+    ) -> Extent:
         """Create an Extent based on the datetimes and bboxes of a list of items.
 
         Args:
@@ -359,7 +365,7 @@ class Extent:
 
         Returns:
             Extent: An Extent that spatially and temporally covers all of the
-                given items.
+            given items.
         """
         bounds_values: List[List[float]] = [
             [float("inf")],
@@ -505,7 +511,7 @@ class Collection(Catalog):
         catalog_type: Optional[CatalogType] = None,
         license: str = "proprietary",
         keywords: Optional[List[str]] = None,
-        providers: Optional[List["Provider_Type"]] = None,
+        providers: Optional[List[Provider]] = None,
         summaries: Optional[Summaries] = None,
         assets: Optional[Dict[str, Asset]] = None,
     ):
@@ -544,7 +550,7 @@ class Collection(Catalog):
 
     def add_item(
         self,
-        item: "Item_Type",
+        item: Item,
         title: Optional[str] = None,
         strategy: Optional[HrefLayoutStrategy] = None,
     ) -> None:
@@ -559,11 +565,11 @@ class Collection(Catalog):
         )
         d["extent"] = self.extent.to_dict()
         d["license"] = self.license
-        if self.stac_extensions is not None:
+        if self.stac_extensions:
             d["stac_extensions"] = self.stac_extensions
-        if self.keywords is not None:
+        if self.keywords:
             d["keywords"] = self.keywords
-        if self.providers is not None:
+        if self.providers:
             d["providers"] = list(map(lambda x: x.to_dict(), self.providers))
         if not self.summaries.is_empty():
             d["summaries"] = self.summaries.to_dict()
@@ -572,7 +578,7 @@ class Collection(Catalog):
 
         return d
 
-    def clone(self) -> "Collection":
+    def clone(self) -> Collection:
         cls = self.__class__
         clone = cls(
             id=self.id,
@@ -606,13 +612,15 @@ class Collection(Catalog):
 
     @classmethod
     def from_dict(
-        cls,
+        cls: Type[C],
         d: Dict[str, Any],
         href: Optional[str] = None,
         root: Optional[Catalog] = None,
         migrate: bool = False,
         preserve_dict: bool = True,
-    ) -> "Collection":
+    ) -> C:
+        from pystac.extensions.version import CollectionVersionExtension
+
         if migrate:
             info = identify_stac_object(d)
             d = migrate_to_latest(d, info)
@@ -629,19 +637,19 @@ class Collection(Catalog):
         description = d.pop("description")
         license = d.pop("license")
         extent = Extent.from_dict(d.pop("extent"))
-        title = d.get("title")
-        stac_extensions = d.get("stac_extensions")
-        keywords = d.get("keywords")
-        providers = d.get("providers")
+        title = d.pop("title", None)
+        stac_extensions = d.pop("stac_extensions", None)
+        keywords = d.pop("keywords", None)
+        providers = d.pop("providers", None)
         if providers is not None:
             providers = list(map(lambda x: pystac.Provider.from_dict(x), providers))
-        summaries = d.get("summaries")
+        summaries = d.pop("summaries", None)
         if summaries is not None:
             summaries = Summaries(summaries)
 
-        assets: Optional[Dict[str, Any]] = {
-            k: Asset.from_dict(v) for k, v in d.get("assets", {}).items()
-        }
+        assets = d.pop("assets", None)
+        if assets:
+            assets = {k: Asset.from_dict(v) for k, v in assets.items()}
         links = d.pop("links")
 
         d.pop("stac_version")
@@ -673,15 +681,45 @@ class Collection(Catalog):
         if root:
             collection.set_root(root)
 
+        try:
+            version = CollectionVersionExtension.ext(collection)
+            if version.deprecated:
+                warnings.warn(
+                    f"The collection '{collection.id}' is deprecated.",
+                    DeprecatedWarning,
+                )
+            # Collection asset deprecation checks pending version extension support
+        except ExtensionNotImplemented:
+            pass
+
         return collection
 
-    def get_assets(self) -> Dict[str, Asset]:
-        """Get this item's assets.
+    def get_assets(
+        self,
+        media_type: Optional[Union[str, pystac.MediaType]] = None,
+        role: Optional[str] = None,
+    ) -> Dict[str, Asset]:
+        """Get this collection's assets.
+
+        Args:
+            media_type: If set, filter the assets such that only those with a
+                matching ``media_type`` are returned.
+            role: If set, filter the assets such that only those with a matching
+                ``role`` are returned.
 
         Returns:
-            Dict[str, Asset]: A copy of the dictionary of this item's assets.
+            Dict[str, Asset]: A dictionary of assets that match ``media_type``
+            and/or ``role`` if set or else all of this collection's assets.
         """
-        return dict(self.assets.items())
+        if media_type is None and role is None:
+            return dict(self.assets.items())
+        assets = dict()
+        for key, asset in self.assets.items():
+            if (media_type is None or asset.media_type == media_type) and (
+                role is None or asset.has_role(role)
+            ):
+                assets[key] = asset
+        return assets
 
     def add_asset(self, key: str, asset: Asset) -> None:
         """Adds an Asset to this item.
@@ -701,13 +739,13 @@ class Collection(Catalog):
 
     def full_copy(
         self, root: Optional["Catalog"] = None, parent: Optional["Catalog"] = None
-    ) -> "Collection":
+    ) -> Collection:
         return cast(Collection, super().full_copy(root, parent))
 
     @classmethod
     def from_file(
-        cls, href: str, stac_io: Optional[pystac.StacIO] = None
-    ) -> "Collection":
+        cls: Type[C], href: str, stac_io: Optional[pystac.StacIO] = None
+    ) -> C:
         result = super().from_file(href, stac_io)
         if not isinstance(result, Collection):
             raise pystac.STACTypeError(f"{result} is not a {Collection}.")
