@@ -1,5 +1,7 @@
 """Implements the :stac-ext:`Electro-Optical Extension <eo>`."""
 
+from __future__ import annotations
+
 from typing import (
     Any,
     Dict,
@@ -14,15 +16,15 @@ from typing import (
 )
 
 import pystac
-from pystac.summaries import RangeSummary
+from pystac.extensions import projection, view
 from pystac.extensions.base import (
     ExtensionManagementMixin,
     PropertiesExtension,
     SummariesExtension,
 )
 from pystac.extensions.hooks import ExtensionHooks
-from pystac.extensions import view, projection
 from pystac.serialization.identify import STACJSONDescription, STACVersionID
+from pystac.summaries import RangeSummary
 from pystac.utils import get_required, map_opt
 
 T = TypeVar("T", pystac.Item, pystac.Asset)
@@ -86,7 +88,7 @@ class Band:
         center_wavelength: Optional[float] = None,
         full_width_half_max: Optional[float] = None,
         solar_illumination: Optional[float] = None,
-    ) -> "Band":
+    ) -> Band:
         """
         Creates a new band.
 
@@ -146,8 +148,8 @@ class Band:
 
     @property
     def description(self) -> Optional[str]:
-        """Get or sets the description to fully explain the band. CommonMark 0.29 syntax MAY be
-        used for rich text representation.
+        """Get or sets the description to fully explain the band. CommonMark 0.29
+        syntax MAY be used for rich text representation.
 
         Returns:
             str
@@ -296,8 +298,8 @@ class EOExtension(
     def apply(
         self, bands: Optional[List[Band]] = None, cloud_cover: Optional[float] = None
     ) -> None:
-        """Applies Electro-Optical Extension properties to the extended :class:`~pystac.Item` or
-        :class:`~pystac.Asset`.
+        """Applies Electro-Optical Extension properties to the extended
+        :class:`~pystac.Item` or :class:`~pystac.Asset`.
 
         Args:
             bands : A list of available bands where each item is a :class:`~Band`
@@ -348,9 +350,9 @@ class EOExtension(
         return SCHEMA_URI
 
     @classmethod
-    def ext(cls, obj: T, add_if_missing: bool = False) -> "EOExtension[T]":
-        """Extends the given STAC Object with properties from the :stac-ext:`Electro-Optical
-        Extension <eo>`.
+    def ext(cls, obj: T, add_if_missing: bool = False) -> EOExtension[T]:
+        """Extends the given STAC Object with properties from the
+        :stac-ext:`Electro-Optical Extension <eo>`.
 
         This extension can be applied to instances of :class:`~pystac.Item` or
         :class:`~pystac.Asset`.
@@ -373,7 +375,7 @@ class EOExtension(
     @classmethod
     def summaries(
         cls, obj: pystac.Collection, add_if_missing: bool = False
-    ) -> "SummariesEOExtension":
+    ) -> SummariesEOExtension:
         """Returns the extended summaries object for the given collection."""
         cls.validate_has_extension(obj, add_if_missing)
         return SummariesEOExtension(obj)
@@ -543,7 +545,7 @@ class EOExtensionHooks(ExtensionHooks):
                         obj["stac_extensions"] = []
                     if view.SCHEMA_URI not in obj["stac_extensions"]:
                         obj["stac_extensions"].append(view.SCHEMA_URI)
-                    if not "view:{}".format(field) in obj["properties"]:
+                    if "view:{}".format(field) not in obj["properties"]:
                         obj["properties"]["view:{}".format(field)] = obj["properties"][
                             "eo:{}".format(field)
                         ]
