@@ -1,19 +1,21 @@
 """Implements the :stac-ext:`Projection Extension <projection>`."""
 
+from __future__ import annotations
+
 import json
 from typing import Any, Dict, Generic, Iterable, List, Optional, TypeVar, Union, cast
 
 import pystac
-from pystac.extensions.hooks import ExtensionHooks
 from pystac.extensions.base import (
     ExtensionManagementMixin,
     PropertiesExtension,
     SummariesExtension,
 )
+from pystac.extensions.hooks import ExtensionHooks
 
 T = TypeVar("T", pystac.Item, pystac.Asset)
 
-SCHEMA_URI: str = "https://stac-extensions.github.io/projection/v1.0.0/schema.json"
+SCHEMA_URI: str = "https://stac-extensions.github.io/projection/v1.1.0/schema.json"
 PREFIX: str = "proj:"
 
 # Field names
@@ -108,8 +110,8 @@ class ProjectionExtension(
 
     @property
     def wkt2(self) -> Optional[str]:
-        """Get or sets the WKT2 string representing the Coordinate Reference System (CRS)
-        that the ``proj:geometry`` and ``proj:bbox`` fields represent
+        """Get or sets the WKT2 string representing the Coordinate Reference System
+        (CRS) that the ``proj:geometry`` and ``proj:bbox`` fields represent
 
         This value is a
         `WKT2 string <https://docs.opengeospatial.org/is/12-063r5/12-063r5.html>`_.
@@ -125,8 +127,8 @@ class ProjectionExtension(
 
     @property
     def projjson(self) -> Optional[Dict[str, Any]]:
-        """Get or sets the PROJJSON string representing the Coordinate Reference System (CRS)
-        that the ``proj:geometry`` and ``proj:bbox`` fields represent
+        """Get or sets the PROJJSON string representing the Coordinate Reference System
+        (CRS) that the ``proj:geometry`` and ``proj:bbox`` fields represent
 
         This value is a
         `PROJJSON object <https://proj.org/specifications/projjson.html>`_.
@@ -184,8 +186,8 @@ class ProjectionExtension(
 
     @property
     def bbox(self) -> Optional[List[float]]:
-        """Get or sets the bounding box of the assets represented by this item in the asset
-        data CRS.
+        """Get or sets the bounding box of the assets represented by this item in the
+        asset data CRS.
 
         Specified as 4 or 6 coordinates based on the CRS defined in the ``epsg``,
         ``projjson`` or ``wkt2`` properties. First two numbers are coordinates of the
@@ -203,7 +205,8 @@ class ProjectionExtension(
 
     @property
     def centroid(self) -> Optional[Dict[str, float]]:
-        """Get or sets coordinates representing the centroid of the item in the asset data CRS.
+        """Get or sets coordinates representing the centroid of the item in the asset
+        data CRS.
 
         Coordinates are defined in latitude and longitude, even if the data coordinate
         system does not use lat/long.
@@ -257,7 +260,7 @@ class ProjectionExtension(
         return SCHEMA_URI
 
     @classmethod
-    def ext(cls, obj: T, add_if_missing: bool = False) -> "ProjectionExtension[T]":
+    def ext(cls, obj: T, add_if_missing: bool = False) -> ProjectionExtension[T]:
         """Extends the given STAC Object with properties from the :stac-ext:`Projection
         Extension <projection>`.
 
@@ -282,16 +285,16 @@ class ProjectionExtension(
     @classmethod
     def summaries(
         cls, obj: pystac.Collection, add_if_missing: bool = False
-    ) -> "SummariesProjectionExtension":
+    ) -> SummariesProjectionExtension:
         """Returns the extended summaries object for the given collection."""
         cls.validate_has_extension(obj, add_if_missing)
         return SummariesProjectionExtension(obj)
 
 
 class ItemProjectionExtension(ProjectionExtension[pystac.Item]):
-    """A concrete implementation of :class:`ProjectionExtension` on an :class:`~pystac.Item`
-    that extends the properties of the Item to include properties defined in the
-    :stac-ext:`Projection Extension <projection>`.
+    """A concrete implementation of :class:`ProjectionExtension` on an
+    :class:`~pystac.Item` that extends the properties of the Item to include properties
+    defined in the :stac-ext:`Projection Extension <projection>`.
 
     This class should generally not be instantiated directly. Instead, call
     :meth:`ProjectionExtension.ext` on an :class:`~pystac.Item` to extend it.
