@@ -1,5 +1,7 @@
 """Implements the :stac-ext:`Raster Extension <raster>`."""
 
+from __future__ import annotations
+
 from typing import Any, Dict, Iterable, List, Optional, Union
 
 import pystac
@@ -88,7 +90,7 @@ class Statistics:
         mean: Optional[float] = None,
         stddev: Optional[float] = None,
         valid_percent: Optional[float] = None,
-    ) -> "Statistics":
+    ) -> Statistics:
         """
         Creates a new band.
 
@@ -198,7 +200,7 @@ class Statistics:
         return self.properties
 
     @staticmethod
-    def from_dict(d: Dict[str, Any]) -> "Statistics":
+    def from_dict(d: Dict[str, Any]) -> Statistics:
         """Constructs an Statistics from a dict.
 
         Returns:
@@ -208,7 +210,8 @@ class Statistics:
 
 
 class Histogram:
-    """Represents pixel distribution information attached to a band in the raster extension.
+    """Represents pixel distribution information attached to a band in the raster
+    extension.
 
     Use Band.create to create a new Band.
     """
@@ -249,7 +252,7 @@ class Histogram:
         min: float,
         max: float,
         buckets: List[int],
-    ) -> "Histogram":
+    ) -> Histogram:
         """
         Creates a new band.
 
@@ -333,7 +336,7 @@ class Histogram:
         return self.properties
 
     @staticmethod
-    def from_dict(d: Dict[str, Any]) -> "Histogram":
+    def from_dict(d: Dict[str, Any]) -> Histogram:
         """Constructs an Histogram from a dict.
 
         Returns:
@@ -416,7 +419,7 @@ class RasterBand:
         scale: Optional[float] = None,
         offset: Optional[float] = None,
         histogram: Optional[Histogram] = None,
-    ) -> "RasterBand":
+    ) -> RasterBand:
         """
         Creates a new band.
 
@@ -525,7 +528,8 @@ class RasterBand:
 
     @property
     def spatial_resolution(self) -> Optional[float]:
-        """Get or sets the average spatial resolution (in meters) of the pixels in the band.
+        """Get or sets the average spatial resolution (in meters) of the pixels in the
+        band.
 
         Returns:
             [float]
@@ -541,7 +545,8 @@ class RasterBand:
 
     @property
     def statistics(self) -> Optional[Statistics]:
-        """Get or sets the average spatial resolution (in meters) of the pixels in the band.
+        """Get or sets the average spatial resolution (in meters) of the pixels in the
+        band.
 
         Returns:
             [Statistics]
@@ -607,7 +612,8 @@ class RasterBand:
 
     @property
     def histogram(self) -> Optional[Histogram]:
-        """Get or sets the histogram distribution information of the pixels values in the band
+        """Get or sets the histogram distribution information of the pixels values in
+        the band.
 
         Returns:
             [Histogram]
@@ -678,9 +684,9 @@ class RasterExtension(
 
     @property
     def bands(self) -> Optional[List[RasterBand]]:
-        """Gets or sets a list of available bands where each item is a :class:`~RasterBand`
-        object (or ``None`` if no bands have been set). If not available the field
-        should not be provided.
+        """Gets or sets a list of available bands where each item is a
+        :class:`~RasterBand` object (or ``None`` if no bands have been set). If not
+        available the field should not be provided.
         """
         return self._get_bands()
 
@@ -701,7 +707,7 @@ class RasterExtension(
         return SCHEMA_URI
 
     @classmethod
-    def ext(cls, obj: pystac.Asset, add_if_missing: bool = False) -> "RasterExtension":
+    def ext(cls, obj: pystac.Asset, add_if_missing: bool = False) -> RasterExtension:
         """Extends the given STAC Object with properties from the :stac-ext:`Raster
         Extension <raster>`.
 
@@ -722,7 +728,7 @@ class RasterExtension(
     @classmethod
     def summaries(
         cls, obj: pystac.Collection, add_if_missing: bool = False
-    ) -> "SummariesRasterExtension":
+    ) -> SummariesRasterExtension:
         cls.validate_has_extension(obj, add_if_missing)
         return SummariesRasterExtension(obj)
 
