@@ -1,28 +1,37 @@
-from copy import copy, deepcopy
-from datetime import datetime as Datetime
-from pystac.catalog import Catalog
-from typing import Any, Dict, List, Optional, Union, cast
+from __future__ import annotations
 
-import dateutil.parser
+import warnings
+from copy import copy, deepcopy
+from html import escape
+from typing import TYPE_CHECKING, Any, Dict, List, Optional, Type, TypeVar, Union, cast
 
 import pystac
 from pystac import STACError, STACObjectType
 from pystac.asset import Asset
+from pystac.catalog import Catalog
+from pystac.collection import Collection
+from pystac.errors import DeprecatedWarning, ExtensionNotImplemented
+from pystac.html.jinja_env import get_jinja_env
 from pystac.link import Link
 from pystac.serialization import (
-    identify_stac_object_type,
     identify_stac_object,
+    identify_stac_object_type,
     migrate_to_latest,
 )
 from pystac.stac_object import STACObject
 from pystac.utils import (
+    datetime_to_str,
     is_absolute_href,
     make_absolute_href,
     make_relative_href,
-    datetime_to_str,
     str_to_datetime,
 )
-from pystac.collection import Collection
+
+T = TypeVar("T", bound="Item")
+
+if TYPE_CHECKING:
+    # avoids conflicts since there are also kwargs and attrs called `datetime`
+    from datetime import datetime as Datetime
 
 
 class Item(STACObject):
@@ -39,9 +48,13 @@ class Item(STACObject):
             using either 2D or 3D geometries. The length of the array must be 2*n
             where n is the number of dimensions. Could also be None in the case of a
             null geometry.
-        datetime : Datetime associated with this item. If None,
-            a start_datetime and end_datetime must be supplied in the properties.
+        datetime : datetime associated with this item. If None,
+            a start_datetime and end_datetime must be supplied.
         properties : A dictionary of additional metadata for the item.
+        start_datetime : Optional start datetime, part of common metadata. This value
+            will override any `start_datetime` key in properties.
+        end_datetime : Optional end datetime, part of common metadata. This value
+            will override any `end_datetime` key in properties.
         stac_extensions : Optional list of extensions the Item implements.
         href : Optional HREF for this item, which be set as the item's
             self link's HREF.
@@ -49,33 +62,52 @@ class Item(STACObject):
             belongs to.
         extra_fields : Extra fields that are part of the top-level JSON
             properties of the Item.
-
-    Attributes:
-        id : Provider identifier. Unique within the STAC.
-        geometry : Defines the full footprint of the asset represented by this
-            item, formatted according to
-            `RFC 7946, section 3.1 (GeoJSON) <https://tools.ietf.org/html/rfc7946>`_.
-        bbox :  Bounding Box of the asset represented by this item
-            using either 2D or 3D geometries. The length of the array is 2*n where n
-            is the number of dimensions. Could also be None in the case of a null
-            geometry.
-        datetime : Datetime associated with this item. If None,
-            the start_datetime and end_datetime in the common_metadata
-            will supply the datetime range of the Item.
-        properties : A dictionary of additional metadata for the item.
-        stac_extensions : Optional list of extensions the Item
-            implements.
-        collection : Collection that this item is a part of.
-        links : A list of :class:`~pystac.Link` objects representing
-            all links associated with this STACObject.
-        assets : Dictionary of asset objects that can be downloaded,
-            each with a unique key.
-        collection_id : The Collection ID that this item belongs to, if
-            any.
-        extra_fields : Extra fields that are part of the top-level JSON
-            properties of the Item.
+        assets : A dictionary mapping string keys to :class:`~pystac.Asset` objects. All
+            :class:`~pystac.Asset` values in the dictionary will have their
+            :attr:`~pystac.Asset.owner` attribute set to the created Item.
     """
 
+    assets: Dict[str, Asset]
+    """Dictionary of :class:`~pystac.Asset` objects, each with a unique key."""
+
+    bbox: Optional[List[float]]
+    """Bounding Box of the asset represented by this item using either 2D or 3D
+    geometries. The length of the array is 2*n where n is the number of dimensions.
+    Could also be None in the case of a null geometry."""
+
+    collection: Optional[Collection]
+    """:class:`~pystac.Collection` to which this Item belongs, if any."""
+
+    collection_id: Optional[str]
+    """The Collection ID that this item belongs to, if any."""
+
+    datetime: Optional[Datetime]
+    """Datetime associated with this item. If ``None``, then
+    :attr:`~pystac.CommonMetadata.start_datetime` and
+    :attr:`~pystac.CommonMetadata.end_datetime` in :attr:`~pystac.Item.common_metadata`
+    will supply the datetime range of the Item."""
+
+    extra_fields: Dict[str, Any]
+    """Extra fields that are part of the top-level JSON fields the Item."""
+
+    geometry: Optional[Dict[str, Any]]
+    """Defines the full footprint of the asset represented by this item, formatted
+    according to `RFC 7946, section 3.1 (GeoJSON)
+    <https://tools.ietf.org/html/rfc7946>`_."""
+
+    id: str
+    """Provider identifier. Unique within the STAC."""
+
+    links: List[Link]
+    """A list of :class:`~pystac.Link` objects representing all links associated with
+    this Item."""
+
+    properties: Dict[str, Any]
+    """A dictionary of additional metadata for the Item."""
+
+    stac_extensions: List[str]
+    """List of extensions the Item implements."""
+
     STAC_OBJECT_TYPE = STACObjectType.ITEM
 
     def __init__(
@@ -85,10 +117,13 @@ class Item(STACObject):
         bbox: Optional[List[float]],
         datetime: Optional[Datetime],
         properties: Dict[str, Any],
+        start_datetime: Optional[Datetime] = None,
+        end_datetime: Optional[Datetime] = None,
         stac_extensions: Optional[List[str]] = None,
         href: Optional[str] = None,
         collection: Optional[Union[str, Collection]] = None,
         extra_fields: Optional[Dict[str, Any]] = None,
+        assets: Optional[Dict[str, Asset]] = None,
     ):
         super().__init__(stac_extensions or [])
 
@@ -104,13 +139,16 @@ class Item(STACObject):
         self.assets: Dict[str, Asset] = {}
 
         self.datetime: Optional[Datetime] = None
+        if start_datetime:
+            properties["start_datetime"] = datetime_to_str(start_datetime)
+        if end_datetime:
+            properties["end_datetime"] = datetime_to_str(end_datetime)
         if datetime is None:
             if "start_datetime" not in properties or "end_datetime" not in properties:
                 raise STACError(
                     "Invalid Item: If datetime is None, "
                     "a start_datetime and end_datetime "
-                    "must be supplied in "
-                    "the properties."
+                    "must be supplied."
                 )
             self.datetime = None
         else:
@@ -126,9 +164,22 @@ class Item(STACObject):
             else:
                 self.collection_id = collection
 
+        self.assets = {}
+        if assets is not None:
+            for k, asset in assets.items():
+                self.add_asset(k, asset)
+
     def __repr__(self) -> str:
         return "<Item id={}>".format(self.id)
 
+    def _repr_html_(self) -> str:
+        jinja_env = get_jinja_env()
+        if jinja_env:
+            template = jinja_env.get_template("Item.jinja2")
+            return str(template.render(item=self))
+        else:
+            return escape(repr(self))
+
     def set_self_href(self, href: Optional[str]) -> None:
         """Sets the absolute HREF that is represented by the ``rel == 'self'``
         :class:`~pystac.Link`.
@@ -188,13 +239,32 @@ class Item(STACObject):
         else:
             asset.extra_fields["datetime"] = datetime_to_str(datetime)
 
-    def get_assets(self) -> Dict[str, Asset]:
+    def get_assets(
+        self,
+        media_type: Optional[Union[str, pystac.MediaType]] = None,
+        role: Optional[str] = None,
+    ) -> Dict[str, Asset]:
         """Get this item's assets.
 
+        Args:
+            media_type: If set, filter the assets such that only those with a
+                matching ``media_type`` are returned.
+            role: If set, filter the assets such that only those with a matching
+                ``role`` are returned.
+
         Returns:
-            Dict[str, Asset]: A copy of the dictionary of this item's assets.
+            Dict[str, Asset]: A dictionary of assets that match ``media_type``
+                and/or ``role`` if set or else all of this item's assets.
         """
-        return dict(self.assets.items())
+        if media_type is None and role is None:
+            return dict(self.assets.items())
+        assets = dict()
+        for key, asset in self.assets.items():
+            if (media_type is None or asset.media_type == media_type) and (
+                role is None or asset.has_role(role)
+            ):
+                assets[key] = asset
+        return assets
 
     def add_asset(self, key: str, asset: Asset) -> None:
         """Adds an Asset to this item.
@@ -206,7 +276,7 @@ class Item(STACObject):
         asset.set_owner(self)
         self.assets[key] = asset
 
-    def make_asset_hrefs_relative(self) -> "Item":
+    def make_asset_hrefs_relative(self) -> Item:
         """Modify each asset's HREF to be relative to this item's self HREF.
 
         Returns:
@@ -227,7 +297,7 @@ class Item(STACObject):
                 asset.href = make_relative_href(asset.href, self_href)
         return self
 
-    def make_asset_hrefs_absolute(self) -> "Item":
+    def make_asset_hrefs_absolute(self) -> Item:
         """Modify each asset's HREF to be absolute.
 
         Any asset HREFs that are relative will be modified to absolute based on this
@@ -251,7 +321,7 @@ class Item(STACObject):
 
         return self
 
-    def set_collection(self, collection: Optional[Collection]) -> "Item":
+    def set_collection(self, collection: Optional[Collection]) -> Item:
         """Set the collection of this item.
 
         This method will replace any existing Collection link and attribute for
@@ -285,7 +355,9 @@ class Item(STACObject):
         else:
             return cast(Collection, collection_link.resolve_stac_object().target)
 
-    def to_dict(self, include_self_link: bool = True) -> Dict[str, Any]:
+    def to_dict(
+        self, include_self_link: bool = True, transform_hrefs: bool = True
+    ) -> Dict[str, Any]:
         links = self.links
         if not include_self_link:
             links = [x for x in links if x.rel != pystac.RelType.SELF]
@@ -303,7 +375,7 @@ class Item(STACObject):
             "id": self.id,
             "properties": self.properties,
             "geometry": self.geometry,
-            "links": [link.to_dict() for link in links],
+            "links": [link.to_dict(transform_href=transform_hrefs) for link in links],
             "assets": assets,
         }
 
@@ -321,7 +393,7 @@ class Item(STACObject):
 
         return d
 
-    def clone(self) -> "Item":
+    def clone(self) -> Item:
         cls = self.__class__
         clone = cls(
             id=self.id,
@@ -331,13 +403,11 @@ class Item(STACObject):
             properties=deepcopy(self.properties),
             stac_extensions=deepcopy(self.stac_extensions),
             collection=self.collection_id,
+            assets={k: asset.clone() for k, asset in self.assets.items()},
         )
         for link in self.links:
             clone.add_link(link.clone())
 
-        for k, asset in self.assets.items():
-            clone.add_asset(k, asset.clone())
-
         return clone
 
     def _object_links(self) -> List[Union[str, pystac.RelType]]:
@@ -348,13 +418,18 @@ class Item(STACObject):
 
     @classmethod
     def from_dict(
-        cls,
+        cls: Type[T],
         d: Dict[str, Any],
         href: Optional[str] = None,
         root: Optional[Catalog] = None,
         migrate: bool = False,
         preserve_dict: bool = True,
-    ) -> "Item":
+    ) -> T:
+        from pystac.extensions.version import ItemVersionExtension
+
+        if preserve_dict:
+            d = deepcopy(d)
+
         if migrate:
             info = identify_stac_object(d)
             d = migrate_to_latest(d, info)
@@ -364,34 +439,41 @@ class Item(STACObject):
                 f"{d} does not represent a {cls.__name__} instance"
             )
 
-        if preserve_dict:
-            d = deepcopy(d)
-
-        id = d.pop("id")
-        geometry = d.pop("geometry")
-        properties = d.pop("properties")
-        bbox = d.pop("bbox", None)
-        stac_extensions = d.get("stac_extensions")
-        collection_id = d.pop("collection", None)
+        # some fields are passed through to __init__
+        pass_through_fields = [
+            "id",
+            "geometry",
+            "bbox",
+            "stac_extensions",
+            "collection",
+        ]
 
+        # some fields need some parsing or special handling
+        parse_fields = ["links", "assets", "properties"]
+        links = d.get("links", [])
+        assets = d.get("assets", {})
+        properties = d.get("properties", {})
         datetime = properties.get("datetime")
         if datetime is not None:
-            datetime = dateutil.parser.parse(datetime)
-        links = d.pop("links")
-        assets = d.pop("assets")
+            datetime = str_to_datetime(datetime)
 
-        d.pop("type")
-        d.pop("stac_version")
+        # some fields are excluded from __init__ entirely
+        exclude_fields = ["type", "stac_version"]
+
+        # get all the fields that are not passed through, parsed, or excluded
+        extra_fields = {
+            k: v
+            for k, v in d.items()
+            if k not in [*pass_through_fields, *parse_fields, *exclude_fields]
+        }
 
         item = cls(
-            id=id,
-            geometry=geometry,
-            bbox=bbox,
+            **{k: d.get(k) for k in pass_through_fields},  # type: ignore
             datetime=datetime,
             properties=properties,
-            stac_extensions=stac_extensions,
-            collection=collection_id,
-            extra_fields=d,
+            extra_fields=extra_fields,
+            href=href,
+            assets={k: Asset.from_dict(v) for k, v in assets.items()},
         )
 
         has_self_link = False
@@ -402,14 +484,20 @@ class Item(STACObject):
         if not has_self_link and href is not None:
             item.add_link(Link.self_href(href))
 
-        for k, v in assets.items():
-            asset = Asset.from_dict(v)
-            asset.set_owner(item)
-            item.assets[k] = asset
-
         if root:
             item.set_root(root)
 
+        try:
+            version = ItemVersionExtension.ext(item)
+            if version.deprecated:
+                warnings.warn(
+                    f"The item '{item.id}' is deprecated.",
+                    DeprecatedWarning,
+                )
+            # Item asset deprecation checks pending version extension support
+        except ExtensionNotImplemented:
+            pass
+
         return item
 
     @property
@@ -419,12 +507,14 @@ class Item(STACObject):
         return pystac.CommonMetadata(self)
 
     def full_copy(
-        self, root: Optional["Catalog"] = None, parent: Optional["Catalog"] = None
-    ) -> "Item":
+        self, root: Optional[Catalog] = None, parent: Optional[Catalog] = None
+    ) -> Item:
         return cast(Item, super().full_copy(root, parent))
 
     @classmethod
-    def from_file(cls, href: str, stac_io: Optional[pystac.StacIO] = None) -> "Item":
+    def from_file(
+        cls: Type[T], href: str, stac_io: Optional[pystac.StacIO] = None
+    ) -> T:
         result = super().from_file(href, stac_io)
         if not isinstance(result, Item):
             raise pystac.STACTypeError(f"{result} is not a {Item}.")
@@ -432,4 +522,21 @@ class Item(STACObject):
 
     @classmethod
     def matches_object_type(cls, d: Dict[str, Any]) -> bool:
+        for field in ("type", "stac_version"):
+            if field not in d:
+                raise pystac.STACTypeError(
+                    f"{d} does not represent a {cls.__name__} instance"
+                    f"'{field}' is missing."
+                )
         return identify_stac_object_type(d) == STACObjectType.ITEM
+
+    @property
+    def __geo_interface__(self) -> Optional[Dict[str, Any]]:
+        """Returns this item's geometry.
+
+        https://gist.github.com/sgillies/2217756
+
+        Returns:
+            Dict[str, Any]: The Item's geometry
+        """
+        return deepcopy(self.geometry)
