@@ -1,24 +1,41 @@
-from abc import abstractmethod, ABC
+from __future__ import annotations
+
+import warnings
+from abc import ABC, abstractmethod
 from collections import OrderedDict
 from string import Formatter
-from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union
+from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Union
 
 import pystac
-from pystac.utils import safe_urlparse, join_path_or_url, JoinType
+from pystac.utils import JoinType, join_path_or_url, safe_urlparse
 
 if TYPE_CHECKING:
-    from pystac.stac_object import STACObject as STACObject_Type
-    from pystac.catalog import Catalog as Catalog_Type
-    from pystac.collection import Collection as Collection_Type
-    from pystac.item import Item as Item_Type
+    from pystac.catalog import Catalog
+    from pystac.collection import Collection
+    from pystac.item import Item
+    from pystac.stac_object import STACObject
 
 
 class TemplateError(Exception):
-    """Exception thrown when an error occurs during converting a template
+    """DEPRECATED.
+
+    .. deprecated:: 1.7.0
+        Use :class:`pystac.errors.TemplateError` instead.
+
+    Exception thrown when an error occurs during converting a template
     string into data for :class:`~pystac.layout.LayoutTemplate`
     """
 
-    pass
+    def __init__(self, *args, **kwargs) -> None:  # type: ignore
+        warnings.warn(
+            message=(
+                "TemplateError in pystac.layout is deprecated and will be "
+                "removed in pystac version 2.0.0. Use TemplateError in "
+                "pystac.errors instead."
+            ),
+            category=DeprecationWarning,
+        )
+        super().__init__(*args, **kwargs)
 
 
 class LayoutTemplate:
@@ -54,6 +71,10 @@ class LayoutTemplate:
     | ``collection``     | The collection ID of an Item's collection.             |
     +--------------------+--------------------------------------------------------+
 
+    The forward slash (``/``) should be used as path separator in the template
+    string regardless of the system path separator (thus both in POSIX-compliant
+    and Windows environments).
+
     Examples::
 
         # Uses the year, month and day of the item
@@ -71,6 +92,16 @@ class LayoutTemplate:
             will be used in case a value cannot be derived from a stac object.
     """
 
+    template: str
+    """The template string to use."""
+
+    defaults: Dict[str, str]
+    """A dictionary of template vars to values. These values will be used in case a
+    value cannot be derived from a stac object."""
+
+    template_vars: List[str]
+    """List of template vars to use when templating."""
+
     # Special template vars specific to Items
     ITEM_TEMPLATE_VARS = ["date", "year", "month", "day", "collection"]
 
@@ -81,7 +112,7 @@ class LayoutTemplate:
         self.defaults = defaults or {}
 
         # Generate list of template vars
-        template_vars: List[str] = []
+        template_vars = []
         for formatter_parse_result in Formatter().parse(template):
             v = formatter_parse_result[1]
             if v is not None:
@@ -90,9 +121,7 @@ class LayoutTemplate:
                 template_vars.append(v)
         self.template_vars = template_vars
 
-    def _get_template_value(
-        self, stac_object: "STACObject_Type", template_var: str
-    ) -> Any:
+    def _get_template_value(self, stac_object: STACObject, template_var: str) -> Any:
         if template_var in self.ITEM_TEMPLATE_VARS:
             if isinstance(stac_object, pystac.Item):
                 # Datetime
@@ -100,7 +129,7 @@ class LayoutTemplate:
                 if dt is None:
                     dt = stac_object.common_metadata.start_datetime
                 if dt is None:
-                    raise TemplateError(
+                    raise pystac.TemplateError(
                         "Item {} does not have a datetime or "
                         "datetime range set; cannot template {} in {}".format(
                             stac_object, template_var, self.template
@@ -120,12 +149,12 @@ class LayoutTemplate:
                 if template_var == "collection":
                     if stac_object.collection_id is not None:
                         return stac_object.collection_id
-                    raise TemplateError(
+                    raise pystac.TemplateError(
                         f"Item {stac_object} does not have a collection ID set; "
                         f"cannot template {template_var} in {self.template}"
                     )
             else:
-                raise TemplateError(
+                raise pystac.TemplateError(
                     '"{}" cannot be used to template non-Item {} in {}'.format(
                         template_var, stac_object, self.template
                     )
@@ -134,28 +163,23 @@ class LayoutTemplate:
         # Allow dot-notation properties for arbitrary object values.
         props = template_var.split(".")
         prop_source: Optional[Union[pystac.STACObject, Dict[str, Any]]] = None
-        error = TemplateError(
+        error = pystac.TemplateError(
             "Cannot find property {} on {} for template {}".format(
                 template_var, stac_object, self.template
             )
         )
 
         try:
-
             if hasattr(stac_object, props[0]):
                 prop_source = stac_object
 
             if prop_source is None and hasattr(stac_object, "properties"):
-                obj_props: Optional[
-                    Dict[str, Any]
-                ] = stac_object.properties  # type:ignore
+                obj_props: Optional[Dict[str, Any]] = stac_object.properties
                 if obj_props is not None and props[0] in obj_props:
                     prop_source = obj_props
 
             if prop_source is None and hasattr(stac_object, "extra_fields"):
-                extra_fields: Optional[
-                    Dict[str, Any]
-                ] = stac_object.extra_fields  # type:ignore
+                extra_fields: Optional[Dict[str, Any]] = stac_object.extra_fields
                 if extra_fields is not None and props[0] in extra_fields:
                     prop_source = extra_fields
 
@@ -172,14 +196,14 @@ class LayoutTemplate:
                     if not hasattr(v, prop):
                         raise error
                     v = getattr(v, prop)
-        except TemplateError as e:
+        except pystac.TemplateError as e:
             if template_var in self.defaults:
                 return self.defaults[template_var]
             raise e
 
         return v
 
-    def get_template_values(self, stac_object: "STACObject_Type") -> Dict[str, Any]:
+    def get_template_values(self, stac_object: STACObject) -> Dict[str, Any]:
         """Gets a dictionary of template variables to values derived from
         the given stac_object. If the template vars cannot be found in the
         stac object, and defaults was supplied to this template, a default
@@ -191,11 +215,11 @@ class LayoutTemplate:
 
         Returns:
             [dict]: A dictionary with keys being the template variables
-                and values being the respective values based on the given
-                stac object.
+            and values being the respective values based on the given
+            stac object.
 
         Raises:
-            TemplateError: If a value for a template variable cannot be
+            pystac.TemplateError: If a value for a template variable cannot be
                 derived from the stac object and there is no default,
                 this error will be raised.
         """
@@ -203,7 +227,7 @@ class LayoutTemplate:
             [(k, self._get_template_value(stac_object, k)) for k in self.template_vars]
         )
 
-    def substitute(self, stac_object: "STACObject_Type") -> str:
+    def substitute(self, stac_object: STACObject) -> str:
         """Substitutes the values derived from
         :meth:`~pystac.layout.LayoutTemplate.get_template_values` into
         the template string for this template.
@@ -214,11 +238,11 @@ class LayoutTemplate:
 
         Returns:
             str: The original template supplied to this LayoutTemplate
-                with template variables replaced by the values derived
-                from this stac object.
+            with template variables replaced by the values derived
+            from this stac object.
 
         Raises:
-            TemplateError: If a value for a template variable cannot be
+            pystac.TemplateError: If a value for a template variable cannot be
                 derived from the stac object and there is no default,
                 this error will be raised.
         """
@@ -234,7 +258,7 @@ class HrefLayoutStrategy(ABC):
     """Base class for HREF Layout strategies."""
 
     def get_href(
-        self, stac_object: "STACObject_Type", parent_dir: str, is_root: bool = False
+        self, stac_object: STACObject, parent_dir: str, is_root: bool = False
     ) -> str:
         if isinstance(stac_object, pystac.Item):
             return self.get_item_href(stac_object, parent_dir)
@@ -246,19 +270,17 @@ class HrefLayoutStrategy(ABC):
             raise pystac.STACError("Unknown STAC object type {}".format(stac_object))
 
     @abstractmethod
-    def get_catalog_href(
-        self, cat: "Catalog_Type", parent_dir: str, is_root: bool
-    ) -> str:
+    def get_catalog_href(self, cat: Catalog, parent_dir: str, is_root: bool) -> str:
         raise NotImplementedError
 
     @abstractmethod
     def get_collection_href(
-        self, col: "Collection_Type", parent_dir: str, is_root: bool
+        self, col: Collection, parent_dir: str, is_root: bool
     ) -> str:
         raise NotImplementedError
 
     @abstractmethod
-    def get_item_href(self, item: "Item_Type", parent_dir: str) -> str:
+    def get_item_href(self, item: Item, parent_dir: str) -> str:
         raise NotImplementedError
 
 
@@ -283,11 +305,29 @@ class CustomLayoutStrategy(HrefLayoutStrategy):
             :class:`~pystac.layout.BestPracticesLayoutStrategy`
     """
 
+    catalog_func: Optional[Callable[[Catalog, str, bool], str]]
+    """A function that takes a :class:`~pystac.Catalog`, a parent directory, and a
+    boolean specifying whether or not this Catalog is the root. If it is the root, it
+    is usually best to not create a subdirectory and put the Catalog file directly
+    in the parent directory. Must return the string path."""
+
+    collection_func: Optional[Callable[[Collection, str, bool], str]]
+    """A function that is used for collections in the same manner as
+    :attr:`~catalog_func`. This takes the same parameters."""
+
+    fallback_strategy: HrefLayoutStrategy
+    """The fallback strategy to use if a function is not provided for a stac object
+    type. Defaults to :class:`~pystac.layout.BestPracticesLayoutStrategy`."""
+
+    item_func: Optional[Callable[[Item, str], str]]
+    """An optional function that takes an :class:`~pystac.Item` and a parent directory
+    and returns the path to be used for the Item."""
+
     def __init__(
         self,
-        catalog_func: Optional[Callable[["Catalog_Type", str, bool], str]] = None,
-        collection_func: Optional[Callable[["Collection_Type", str, bool], str]] = None,
-        item_func: Optional[Callable[["Item_Type", str], str]] = None,
+        catalog_func: Optional[Callable[[Catalog, str, bool], str]] = None,
+        collection_func: Optional[Callable[[Collection, str, bool], str]] = None,
+        item_func: Optional[Callable[[Item, str], str]] = None,
         fallback_strategy: Optional[HrefLayoutStrategy] = None,
     ):
         self.item_func = item_func
@@ -295,11 +335,9 @@ class CustomLayoutStrategy(HrefLayoutStrategy):
         self.catalog_func = catalog_func
         if fallback_strategy is None:
             fallback_strategy = BestPracticesLayoutStrategy()
-        self.fallback_strategy: HrefLayoutStrategy = fallback_strategy
+        self.fallback_strategy = fallback_strategy
 
-    def get_catalog_href(
-        self, cat: "Catalog_Type", parent_dir: str, is_root: bool
-    ) -> str:
+    def get_catalog_href(self, cat: Catalog, parent_dir: str, is_root: bool) -> str:
         if self.catalog_func is not None:
             result = self.catalog_func(cat, parent_dir, is_root)
             if result is not None:
@@ -307,7 +345,7 @@ class CustomLayoutStrategy(HrefLayoutStrategy):
         return self.fallback_strategy.get_catalog_href(cat, parent_dir, is_root)
 
     def get_collection_href(
-        self, col: "Collection_Type", parent_dir: str, is_root: bool
+        self, col: Collection, parent_dir: str, is_root: bool
     ) -> str:
         if self.collection_func is not None:
             result = self.collection_func(col, parent_dir, is_root)
@@ -315,7 +353,7 @@ class CustomLayoutStrategy(HrefLayoutStrategy):
                 return result
         return self.fallback_strategy.get_collection_href(col, parent_dir, is_root)
 
-    def get_item_href(self, item: "Item_Type", parent_dir: str) -> str:
+    def get_item_href(self, item: Item, parent_dir: str) -> str:
         if self.item_func is not None:
             result = self.item_func(item, parent_dir)
             if result is not None:
@@ -348,6 +386,22 @@ class TemplateLayoutStrategy(HrefLayoutStrategy):
             :class:`~pystac.layout.BestPracticesLayoutStrategy`
     """
 
+    catalog_template: Optional[LayoutTemplate]
+    """The template string to use for catalog paths. Must be a valid template string
+    that can be used by :class:`~pystac.layout.LayoutTemplate`."""
+
+    collection_template: Optional[LayoutTemplate]
+    """The template string to use for collection paths. Must be a valid template string
+    that can be used by :class:`~pystac.layout.LayoutTemplate`."""
+
+    fallback_strategy: HrefLayoutStrategy
+    """The fallback strategy to use if a template is not provided. Defaults to
+    :class:`~pystac.layout.BestPracticesLayoutStrategy`."""
+
+    item_template: Optional[LayoutTemplate]
+    """The template string to use for item paths. Must be a valid template string that
+    can be used by :class:`~pystac.layout.LayoutTemplate`."""
+
     def __init__(
         self,
         catalog_template: Optional[str] = None,
@@ -369,11 +423,9 @@ class TemplateLayoutStrategy(HrefLayoutStrategy):
 
         if fallback_strategy is None:
             fallback_strategy = BestPracticesLayoutStrategy()
-        self.fallback_strategy: HrefLayoutStrategy = fallback_strategy
+        self.fallback_strategy = fallback_strategy
 
-    def get_catalog_href(
-        self, cat: "Catalog_Type", parent_dir: str, is_root: bool
-    ) -> str:
+    def get_catalog_href(self, cat: Catalog, parent_dir: str, is_root: bool) -> str:
         parsed_parent_dir = safe_urlparse(parent_dir)
         join_type = JoinType.from_parsed_uri(parsed_parent_dir)
 
@@ -389,7 +441,7 @@ class TemplateLayoutStrategy(HrefLayoutStrategy):
             return join_path_or_url(join_type, parent_dir, template_path)
 
     def get_collection_href(
-        self, col: "Collection_Type", parent_dir: str, is_root: bool
+        self, col: Collection, parent_dir: str, is_root: bool
     ) -> str:
         parsed_parent_dir = safe_urlparse(parent_dir)
         join_type = JoinType.from_parsed_uri(parsed_parent_dir)
@@ -405,7 +457,7 @@ class TemplateLayoutStrategy(HrefLayoutStrategy):
 
             return join_path_or_url(join_type, parent_dir, template_path)
 
-    def get_item_href(self, item: "Item_Type", parent_dir: str) -> str:
+    def get_item_href(self, item: Item, parent_dir: str) -> str:
         parsed_parent_dir = safe_urlparse(parent_dir)
         join_type = JoinType.from_parsed_uri(parsed_parent_dir)
 
@@ -436,9 +488,7 @@ class BestPracticesLayoutStrategy(HrefLayoutStrategy):
     All paths are appended to the parent directory.
     """
 
-    def get_catalog_href(
-        self, cat: "Catalog_Type", parent_dir: str, is_root: bool
-    ) -> str:
+    def get_catalog_href(self, cat: Catalog, parent_dir: str, is_root: bool) -> str:
         parsed_parent_dir = safe_urlparse(parent_dir)
         join_type = JoinType.from_parsed_uri(parsed_parent_dir)
 
@@ -450,7 +500,7 @@ class BestPracticesLayoutStrategy(HrefLayoutStrategy):
         return join_path_or_url(join_type, cat_root, cat.DEFAULT_FILE_NAME)
 
     def get_collection_href(
-        self, col: "Collection_Type", parent_dir: str, is_root: bool
+        self, col: Collection, parent_dir: str, is_root: bool
     ) -> str:
         parsed_parent_dir = safe_urlparse(parent_dir)
         join_type = JoinType.from_parsed_uri(parsed_parent_dir)
@@ -462,10 +512,46 @@ class BestPracticesLayoutStrategy(HrefLayoutStrategy):
 
         return join_path_or_url(join_type, col_root, col.DEFAULT_FILE_NAME)
 
-    def get_item_href(self, item: "Item_Type", parent_dir: str) -> str:
+    def get_item_href(self, item: Item, parent_dir: str) -> str:
         parsed_parent_dir = safe_urlparse(parent_dir)
         join_type = JoinType.from_parsed_uri(parsed_parent_dir)
 
         item_root = join_path_or_url(join_type, parent_dir, "{}".format(item.id))
 
         return join_path_or_url(join_type, item_root, "{}.json".format(item.id))
+
+
+class AsIsLayoutStrategy(HrefLayoutStrategy):
+    """Layout strategy that simply preserves the current href of all objects.
+
+    If any object doesn't have a self href, a ValueError is raised.
+    """
+
+    def get_catalog_href(self, cat: Catalog, parent_dir: str, is_root: bool) -> str:
+        href = cat.self_href
+        if href is None:
+            raise ValueError(
+                f"Catalog is missing href, required for AsIsLayoutStrategy: {cat}"
+            )
+        else:
+            return href
+
+    def get_collection_href(
+        self, col: Collection, parent_dir: str, is_root: bool
+    ) -> str:
+        href = col.self_href
+        if href is None:
+            raise ValueError(
+                f"Collection is missing href, required for AsIsLayoutStrategy: {col}"
+            )
+        else:
+            return href
+
+    def get_item_href(self, item: Item, parent_dir: str) -> str:
+        href = item.self_href
+        if href is None:
+            raise ValueError(
+                f"Item is missing href, required for AsIsLayoutStrategy: {item}"
+            )
+        else:
+            return href
