@@ -1,5 +1,18 @@
+from __future__ import annotations
+
 from abc import ABC, abstractmethod
-from typing import Any, Dict, Iterable, List, Optional, Type, cast, TYPE_CHECKING, Union
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Dict,
+    Iterable,
+    List,
+    Optional,
+    Type,
+    TypeVar,
+    Union,
+    cast,
+)
 
 import pystac
 from pystac import STACError
@@ -7,7 +20,9 @@ from pystac.link import Link
 from pystac.utils import StringEnum, is_absolute_href, make_absolute_href
 
 if TYPE_CHECKING:
-    from pystac.catalog import Catalog as Catalog_Type
+    from pystac.catalog import Catalog
+
+S = TypeVar("S", bound="STACObject")
 
 
 class STACObjectType(StringEnum):
@@ -81,25 +96,61 @@ class STACObject(ABC):
 
         self.links = [link for link in self.links if link.rel != rel]
 
-    def get_single_link(
-        self,
-        rel: Union[str, pystac.RelType],
-        media_type: Optional[Union[str, pystac.MediaType]] = None,
-    ) -> Optional[Link]:
-        """Get single link that match the given ``rel`` and, optionally,
-        ``media_type``. If ``media_type`` is ``None``, then the link is
-        matched only on the ``rel`` value.
+    def remove_hierarchical_links(self, add_canonical: bool = False) -> List[Link]:
+        """Removes all hierarchical links from this object.
+
+        See :py:const:`pystac.link.HIERARCHICAL_LINKS` for a list of all
+        hierarchical links. If the object has a ``self`` href and
+        ``add_canonical`` is True, a link with ``rel="canonical"`` is added.
 
         Args:
-             rel : The :class:`~pystac.Link` ``rel`` to match on.
-             media_type: The :class:`~pystack.MediaType` ``media_type`` to match on
-        """
+            add_canonical : If true, and this item has a ``self`` href, that
+                href is used to build a ``canonical`` link.
 
+        Returns:
+            List[Link]: All removed links
+        """
+        keep = list()
+        self_href = self.get_self_href()
+        if add_canonical and self_href is not None:
+            keep.append(
+                Link("canonical", self_href, media_type=pystac.MediaType.GEOJSON)
+            )
+        remove = list()
+        for link in self.links:
+            if link.is_hierarchical():
+                remove.append(link)
+            else:
+                keep.append(link)
+        self.links = keep
+        return remove
+
+    def get_single_link(
+        self,
+        rel: Optional[Union[str, pystac.RelType]] = None,
+        media_type: Optional[Union[str, pystac.MediaType]] = None,
+    ) -> Optional[Link]:
+        """Get a single :class:`~pystac.Link` instance associated with this
+        object.
+
+        Args:
+            rel : If set, filter links such that only those
+                matching this relationship are returned.
+            media_type: If set, filter the links such that only
+                those matching media_type are returned
+
+        Returns:
+            Optional[:class:`~pystac.Link`]: First link that matches ``rel``
+            and/or ``media_type``, or else the first link associated with
+            this object.
+        """
+        if rel is None and media_type is None:
+            return next(iter(self.links), None)
         return next(
             (
                 link
                 for link in self.links
-                if link.rel == rel
+                if (rel is None or link.rel == rel)
                 and (media_type is None or link.media_type == media_type)
             ),
             None,
@@ -119,16 +170,17 @@ class STACObject(ABC):
                 those matching media_type are returned
 
         Returns:
-            List[:class:`~pystac.Link`]: A list of links that match ``rel`` if set,
-                or else all links associated with this object.
+            List[:class:`~pystac.Link`]: A list of links that match ``rel`` and/
+            or ``media_type`` if set, or else all links associated with this
+            object.
         """
-        if rel is None:
+        if rel is None and media_type is None:
             return self.links
         else:
             return [
                 link
                 for link in self.links
-                if link.rel == rel
+                if (rel is None or link.rel == rel)
                 and (media_type is None or link.media_type == media_type)
             ]
 
@@ -209,7 +261,7 @@ class STACObject(ABC):
         if root_link is not None and root_link.is_resolved():
             cast(pystac.Catalog, root_link.target)._resolved_objects.cache(self)
 
-    def get_root(self) -> Optional["Catalog_Type"]:
+    def get_root(self) -> Optional[Catalog]:
         """Get the :class:`~pystac.Catalog` or :class:`~pystac.Collection` to
         the root for this object. The root is represented by a
         :class:`~pystac.Link` with ``rel == 'root'``.
@@ -228,7 +280,7 @@ class STACObject(ABC):
         else:
             return None
 
-    def set_root(self, root: Optional["Catalog_Type"]) -> None:
+    def set_root(self, root: Optional[Catalog]) -> None:
         """Sets the root :class:`~pystac.Catalog` or :class:`~pystac.Collection`
         for this object.
 
@@ -264,7 +316,7 @@ class STACObject(ABC):
                 self.add_link(new_root_link)
             root._resolved_objects.cache(self)
 
-    def get_parent(self) -> Optional["Catalog_Type"]:
+    def get_parent(self) -> Optional[Catalog]:
         """Get the :class:`~pystac.Catalog` or :class:`~pystac.Collection` to
         the parent for this object. The root is represented by a
         :class:`~pystac.Link` with ``rel == 'parent'``.
@@ -280,7 +332,7 @@ class STACObject(ABC):
         else:
             return None
 
-    def set_parent(self, parent: Optional["Catalog_Type"]) -> None:
+    def set_parent(self, parent: Optional[Catalog]) -> None:
         """Sets the parent :class:`~pystac.Catalog` or :class:`~pystac.Collection`
         for this object.
 
@@ -294,8 +346,8 @@ class STACObject(ABC):
             self.add_link(Link.parent(parent))
 
     def get_stac_objects(
-        self, rel: Union[str, pystac.RelType], typ: Optional[Type["STACObject"]] = None
-    ) -> Iterable["STACObject"]:
+        self, rel: Union[str, pystac.RelType], typ: Optional[Type[STACObject]] = None
+    ) -> Iterable[STACObject]:
         """Gets the :class:`~pystac.STACObject` instances that are linked to
         by links with their ``rel`` property matching the passed in argument.
 
@@ -316,7 +368,7 @@ class STACObject(ABC):
             if link.rel == rel:
                 link.resolve_stac_object(root=self.get_root())
                 if typ is None or isinstance(link.target, typ):
-                    yield cast("STACObject", link.target)
+                    yield cast(STACObject, link.target)
 
     def save_object(
         self,
@@ -367,9 +419,9 @@ class STACObject(ABC):
 
     def full_copy(
         self,
-        root: Optional["Catalog_Type"] = None,
-        parent: Optional["Catalog_Type"] = None,
-    ) -> "STACObject":
+        root: Optional[Catalog] = None,
+        parent: Optional[Catalog] = None,
+    ) -> STACObject:
         """Create a full copy of this STAC object and any stac objects linked to by
         this object.
 
@@ -381,7 +433,7 @@ class STACObject(ABC):
 
         Returns:
             STACObject: A full copy of this object, as well as any objects this object
-                links to.
+            links to.
         """
         clone = self.clone()
 
@@ -470,7 +522,7 @@ class STACObject(ABC):
         raise NotImplementedError
 
     @abstractmethod
-    def clone(self) -> "STACObject":
+    def clone(self) -> STACObject:
         """Clones this object.
 
         Cloning an object will make a copy of all properties and links of the object;
@@ -485,8 +537,8 @@ class STACObject(ABC):
 
     @classmethod
     def from_file(
-        cls, href: str, stac_io: Optional[pystac.StacIO] = None
-    ) -> "STACObject":
+        cls: Type[S], href: str, stac_io: Optional[pystac.StacIO] = None
+    ) -> S:
         """Reads a STACObject implementation from a file.
 
         Args:
@@ -499,7 +551,7 @@ class STACObject(ABC):
             by the JSON read from the file located at HREF.
         """
         if cls == STACObject:
-            return pystac.read_file(href)
+            return cast(S, pystac.read_file(href))
 
         if stac_io is None:
             stac_io = pystac.StacIO.default()
@@ -525,13 +577,13 @@ class STACObject(ABC):
     @classmethod
     @abstractmethod
     def from_dict(
-        cls,
+        cls: Type[S],
         d: Dict[str, Any],
         href: Optional[str] = None,
-        root: Optional["Catalog_Type"] = None,
+        root: Optional[Catalog] = None,
         migrate: bool = False,
         preserve_dict: bool = True,
-    ) -> "STACObject":
+    ) -> S:
         """Parses this STACObject from the passed in dictionary.
 
         Args:
