@@ -1,23 +1,23 @@
 import json
 import os
-import unittest
 import tempfile
+import unittest
 from typing import List, Union
 
 import pystac
-from pystac import Catalog, Collection, ExtensionTypeError, Item, CatalogType
+import pystac.validation
+from pystac import Catalog, CatalogType, Collection, ExtensionTypeError, Item
 from pystac.extensions.label import (
-    LabelExtension,
     LabelClasses,
     LabelCount,
+    LabelExtension,
     LabelMethod,
     LabelOverview,
+    LabelRelType,
     LabelStatistics,
     LabelTask,
     LabelType,
-    LabelRelType,
 )
-import pystac.validation
 from pystac.utils import get_opt
 from tests.utils import TestCases, assert_to_from_dict
 
@@ -132,13 +132,13 @@ class LabelTest(unittest.TestCase):
         self.assertEqual(len(LabelExtension.ext(label_example_1).label_tasks or []), 2)
 
     def test_get_sources(self) -> None:
-        cat = TestCases.test_case_1()
+        cat = TestCases.case_1()
 
-        items = cat.get_all_items()
+        items = list(cat.get_all_items())
         item_ids = set([i.id for i in items])
 
         for li in items:
-            if LabelExtension.ext(li).has_extension:
+            if LabelExtension.has_extension(li):
                 sources = list(LabelExtension.ext(li).get_sources() or [])
                 self.assertEqual(len(sources), 1)
                 self.assertTrue(sources[0].id in item_ids)
@@ -152,7 +152,7 @@ class LabelTest(unittest.TestCase):
 
         with tempfile.TemporaryDirectory() as tmp_dir:
             cat_dir = os.path.join(tmp_dir, "catalog")
-            catalog = TestCases.test_case_1()
+            catalog = TestCases.case_1()
             catalog.normalize_and_save(cat_dir, catalog_type=CatalogType.SELF_CONTAINED)
 
             cat_read = Catalog.from_file(os.path.join(cat_dir, "catalog.json"))
@@ -163,8 +163,8 @@ class LabelTest(unittest.TestCase):
     def test_read_label_item_owns_asset(self) -> None:
         item = next(
             x
-            for x in TestCases.test_case_2().get_all_items()
-            if LabelExtension.ext(x).has_extension
+            for x in TestCases.case_2().get_all_items()
+            if LabelExtension.has_extension(x)
         )
         assert len(item.assets) > 0
         for asset_key in item.assets:
@@ -196,6 +196,11 @@ class LabelTest(unittest.TestCase):
         # Set
         LabelExtension.ext(label_item).label_type = LabelType.RASTER
         self.assertEqual(LabelType.RASTER, label_item.properties["label:type"])
+        # name for each label:classes object must be null to pass validation
+        label_classes = LabelExtension.ext(label_item).label_classes
+        assert label_classes is not None
+        for classes_obj in label_classes:
+            classes_obj.name = None
         label_item.validate()
 
     def test_label_properties(self) -> None:
@@ -248,6 +253,10 @@ class LabelTest(unittest.TestCase):
 
         label_item.validate()
 
+    def test_label_classes_typing(self) -> None:
+        classes: List[str] = ["foo", "bar"]
+        LabelClasses.create(classes=classes)
+
     def test_label_tasks(self) -> None:
         label_item = pystac.Item.from_file(self.label_example_1_uri)
 
@@ -347,7 +356,6 @@ class LabelTest(unittest.TestCase):
         label_item.validate()
 
     def test_merge_label_overviews(self) -> None:
-
         overview_1 = LabelOverview.create(
             property_key="label",
             counts=[
