@@ -1,22 +1,22 @@
 """Tests for pystac.extensions.sat."""
 
-import datetime
-from pystac.summaries import RangeSummary
-from typing import Any, Dict
 import unittest
+from datetime import datetime
+from typing import Any, Dict
 
 import pystac
-from pystac.utils import str_to_datetime, datetime_to_str
 from pystac import ExtensionTypeError
 from pystac.extensions import sat
 from pystac.extensions.sat import OrbitState, SatExtension
+from pystac.summaries import RangeSummary
+from pystac.utils import datetime_to_str, str_to_datetime
 from tests.utils import TestCases
 
 
 def make_item() -> pystac.Item:
     """Create basic test items that are only slightly different."""
     asset_id = "an/asset"
-    start = datetime.datetime(2018, 1, 2)
+    start = datetime(2018, 1, 2)
     item = pystac.Item(
         id=asset_id, geometry=None, bbox=None, datetime=start, properties={}
     )
