@@ -4,9 +4,9 @@ from datetime import datetime
 
 import pystac
 from pystac import ExtensionTypeError
-from pystac.summaries import RangeSummary
 from pystac.extensions.timestamps import TimestampsExtension
-from pystac.utils import get_opt, str_to_datetime, datetime_to_str
+from pystac.summaries import RangeSummary
+from pystac.utils import datetime_to_str, get_opt, str_to_datetime
 from tests.utils import TestCases, assert_to_from_dict
 
 
@@ -25,7 +25,7 @@ class TimestampsTest(unittest.TestCase):
         assert_to_from_dict(self, pystac.Item, self.item_dict)
 
     def test_apply(self) -> None:
-        item = next(iter(TestCases.test_case_2().get_all_items()))
+        item = next(iter(TestCases.case_2().get_all_items()))
         self.assertFalse(TimestampsExtension.has_extension(item))
 
         TimestampsExtension.add_to(item)
