@@ -1,13 +1,21 @@
 """Tests for pystac.extensions.version."""
 
-import datetime
 import unittest
-from typing import List, Optional
+from datetime import datetime
+from typing import Generator, List, Optional
+
+import pytest
 
 import pystac
-from pystac import ExtensionTypeError
+from pystac.errors import DeprecatedWarning, ExtensionTypeError
 from pystac.extensions import version
-from pystac.extensions.version import VersionExtension, VersionRelType
+from pystac.extensions.version import (
+    CollectionVersionExtension,
+    ItemVersionExtension,
+    VersionExtension,
+    VersionRelType,
+    ignore_deprecated,
+)
 from tests.utils import TestCases
 
 URL_TEMPLATE: str = "http://example.com/catalog/%s.json"
@@ -16,7 +24,7 @@ URL_TEMPLATE: str = "http://example.com/catalog/%s.json"
 def make_item(year: int) -> pystac.Item:
     """Create basic test items that are only slightly different."""
     asset_id = f"USGS/GAP/CONUS/{year}"
-    start = datetime.datetime(year, 1, 2)
+    start = datetime(year, 1, 2)
 
     item = pystac.Item(
         id=asset_id, geometry=None, bbox=None, datetime=start, properties={}
@@ -132,7 +140,7 @@ class ItemVersionExtensionTest(unittest.TestCase):
         self.item.validate()
 
     def test_full_copy(self) -> None:
-        cat = TestCases.test_case_1()
+        cat = TestCases.case_1()
 
         # Fetch two items from the catalog
         item1 = cat.get_item("area-1-1-imagery", recursive=True)
@@ -252,11 +260,11 @@ class ItemVersionExtensionTest(unittest.TestCase):
 
 def make_collection(year: int) -> pystac.Collection:
     asset_id = f"my/collection/of/things/{year}"
-    start = datetime.datetime(2014, 8, 10)
-    end = datetime.datetime(year, 1, 3, 4, 5)
+    start = datetime(2014, 8, 10)
+    end = datetime(year, 1, 3, 4, 5)
     bboxes = [[-180.0, -90.0, 180.0, 90.0]]
     spatial_extent = pystac.SpatialExtent(bboxes)
-    intervals: List[List[Optional[datetime.datetime]]] = [[start, end]]
+    intervals: List[List[Optional[datetime]]] = [[start, end]]
     temporal_extent = pystac.TemporalExtent(intervals)
     extent = pystac.Extent(spatial_extent, temporal_extent)
 
@@ -359,7 +367,7 @@ class CollectionVersionExtensionTest(unittest.TestCase):
         self.collection.validate()
 
     def test_full_copy(self) -> None:
-        cat = TestCases.test_case_1()
+        cat = TestCases.case_1()
 
         # Fetch two collections from the catalog
         col1 = cat.get_child("area-1-1", recursive=True)
@@ -459,17 +467,84 @@ class CollectionVersionExtensionTest(unittest.TestCase):
 
     def test_extension_not_implemented(self) -> None:
         # Should raise exception if Collection does not include extension URI
-        collection = pystac.Collection.from_file(self.example_collection_uri)
+        with ignore_deprecated():
+            collection = pystac.Collection.from_file(self.example_collection_uri)
         collection.stac_extensions.remove(VersionExtension.get_schema_uri())
 
         with self.assertRaises(pystac.ExtensionNotImplemented):
             _ = VersionExtension.ext(collection)
 
     def test_ext_add_to(self) -> None:
-        collection = pystac.Collection.from_file(self.example_collection_uri)
+        with ignore_deprecated():
+            collection = pystac.Collection.from_file(self.example_collection_uri)
         collection.stac_extensions.remove(VersionExtension.get_schema_uri())
         self.assertNotIn(VersionExtension.get_schema_uri(), collection.stac_extensions)
 
         _ = VersionExtension.ext(collection, add_if_missing=True)
 
         self.assertIn(VersionExtension.get_schema_uri(), collection.stac_extensions)
+
+
+def test_item_deprecation_warning(
+    item: pystac.Item, recwarn: Generator[pytest.WarningsRecorder, None, None]
+) -> None:
+    version = ItemVersionExtension.ext(item, add_if_missing=True)
+    version.deprecated = True
+    item_dict = item.to_dict()
+    with pytest.warns(DeprecatedWarning, match="The item 'test-item' is deprecated."):
+        _ = pystac.Item.from_dict(item_dict)
+
+    version.deprecated = False
+    item_dict = item.to_dict()
+    _ = pystac.Item.from_dict(item_dict)
+    assert len(list(recwarn)) == 0
+
+    version.deprecated = None
+    item_dict = item.to_dict()
+    _ = pystac.Item.from_dict(item_dict)
+    assert len(list(recwarn)) == 0
+
+    ItemVersionExtension.remove_from(item)
+    item_dict = item.to_dict()
+    _ = pystac.Item.from_dict(item_dict)
+    assert len(list(recwarn)) == 0
+
+
+def test_collection_deprecation_warning(
+    collection: pystac.Collection,
+    recwarn: Generator[pytest.WarningsRecorder, None, None],
+) -> None:
+    version = CollectionVersionExtension.ext(collection, add_if_missing=True)
+    version.deprecated = True
+    collection_dict = collection.to_dict()
+    with pytest.warns(
+        DeprecatedWarning, match="The collection 'test-collection' is deprecated."
+    ):
+        _ = pystac.Collection.from_dict(collection_dict)
+
+    version.deprecated = False
+    collection.extra_fields["deprecated"] = False
+    collection_dict = collection.to_dict()
+    _ = pystac.Collection.from_dict(collection_dict)
+    assert len(list(recwarn)) == 0
+
+    version.deprecated = None
+    collection_dict = collection.to_dict()
+    _ = pystac.Collection.from_dict(collection_dict)
+    assert len(list(recwarn)) == 0
+
+    CollectionVersionExtension.remove_from(collection)
+    collection_dict = collection.to_dict()
+    _ = pystac.Collection.from_dict(collection_dict)
+    assert len(list(recwarn)) == 0
+
+
+def test_ignore_deprecated_context_manager(
+    item: pystac.Item, recwarn: Generator[pytest.WarningsRecorder, None, None]
+) -> None:
+    version = VersionExtension.ext(item, add_if_missing=True)
+    version.deprecated = True
+    item_dict = item.to_dict()
+    with ignore_deprecated():
+        _ = pystac.Item.from_dict(item_dict)
+    assert len(list(recwarn)) == 0
