@@ -1,25 +1,31 @@
-from copy import deepcopy
-import unittest
-import os
+from __future__ import annotations
+
 import json
-from datetime import datetime
-from dateutil import tz
+import os
 import tempfile
+import unittest
+from copy import deepcopy
+from datetime import datetime
+from typing import Any, Dict, Optional
+
+import pytest
+from dateutil import tz
 
 import pystac
-from pystac.extensions.eo import EOExtension
-from pystac.validation import validate_dict
 from pystac import (
+    Catalog,
+    CatalogType,
     Collection,
-    Item,
     Extent,
+    Item,
+    Provider,
     SpatialExtent,
     TemporalExtent,
-    CatalogType,
-    Provider,
 )
-from pystac.utils import datetime_to_str, get_required
-from tests.utils import TestCases, ARBITRARY_GEOM, ARBITRARY_BBOX
+from pystac.extensions.eo import EOExtension
+from pystac.utils import datetime_to_str, get_required, str_to_datetime
+from pystac.validation import validate_dict
+from tests.utils import ARBITRARY_BBOX, ARBITRARY_GEOM, TestCases
 
 TEST_DATETIME = datetime(2020, 3, 14, 16, 32)
 
@@ -57,13 +63,13 @@ class CollectionTest(unittest.TestCase):
             self.assertTrue(type(x) is float)
 
     def test_read_eo_items_are_heritable(self) -> None:
-        cat = TestCases.test_case_5()
+        cat = TestCases.case_5()
         item = next(iter(cat.get_all_items()))
 
         self.assertTrue(EOExtension.has_extension(item))
 
     def test_save_uses_previous_catalog_type(self) -> None:
-        collection = TestCases.test_case_8()
+        collection = TestCases.case_8()
         assert collection.STAC_OBJECT_TYPE == pystac.STACObjectType.COLLECTION
         self.assertEqual(collection.catalog_type, CatalogType.SELF_CONTAINED)
         with tempfile.TemporaryDirectory() as tmp_dir:
@@ -75,13 +81,26 @@ class CollectionTest(unittest.TestCase):
             self.assertEqual(collection2.catalog_type, CatalogType.SELF_CONTAINED)
 
     def test_clone_uses_previous_catalog_type(self) -> None:
-        catalog = TestCases.test_case_8()
+        catalog = TestCases.case_8()
         assert catalog.catalog_type == CatalogType.SELF_CONTAINED
         clone = catalog.clone()
         self.assertEqual(clone.catalog_type, CatalogType.SELF_CONTAINED)
 
+    def test_clone_cant_mutate_original(self) -> None:
+        collection = TestCases.case_8()
+        assert collection.keywords is not None
+        self.assertListEqual(collection.keywords, ["disaster", "open"])
+        clone = collection.clone()
+        clone.extra_fields["test"] = "extra"
+        self.assertNotIn("test", collection.extra_fields)
+        assert clone.keywords is not None
+        clone.keywords.append("clone")
+        self.assertListEqual(clone.keywords, ["disaster", "open", "clone"])
+        self.assertListEqual(collection.keywords, ["disaster", "open"])
+        self.assertNotEqual(id(collection.summaries), id(clone.summaries))
+
     def test_multiple_extents(self) -> None:
-        cat1 = TestCases.test_case_1()
+        cat1 = TestCases.case_1()
         country = cat1.get_child("country-1")
         assert country is not None
         col1 = country.get_child("area-1-1")
@@ -109,7 +128,7 @@ class CollectionTest(unittest.TestCase):
         self.assertDictEqual(cloned_ext.to_dict(), multi_ext_dict["extent"])
 
     def test_extra_fields(self) -> None:
-        catalog = TestCases.test_case_2()
+        catalog = TestCases.case_2()
         collection = catalog.get_child("1a8c1632-fa91-4a62-b33e-3a87c2ebdf16")
         assert collection is not None
 
@@ -128,8 +147,7 @@ class CollectionTest(unittest.TestCase):
             self.assertEqual(read_col.extra_fields["test"], "extra")
 
     def test_update_extents(self) -> None:
-
-        catalog = TestCases.test_case_2()
+        catalog = TestCases.case_2()
         base_collection = catalog.get_child("1a8c1632-fa91-4a62-b33e-3a87c2ebdf16")
         assert isinstance(base_collection, Collection)
         base_extent = base_collection.extent
@@ -213,6 +231,62 @@ class CollectionTest(unittest.TestCase):
         collection = pystac.Collection.from_dict(data)
         collection.validate()
 
+    def test_get_assets(self) -> None:
+        collection = pystac.Collection.from_file(
+            TestCases.get_path("data-files/collections/with-assets.json")
+        )
+
+        media_type_filter = collection.get_assets(media_type=pystac.MediaType.PNG)
+        self.assertCountEqual(media_type_filter.keys(), ["thumbnail"])
+        role_filter = collection.get_assets(role="thumbnail")
+        self.assertCountEqual(role_filter.keys(), ["thumbnail"])
+        multi_filter = collection.get_assets(
+            media_type=pystac.MediaType.PNG, role="thumbnail"
+        )
+        self.assertCountEqual(multi_filter.keys(), ["thumbnail"])
+
+        no_filter = collection.get_assets()
+        self.assertCountEqual(no_filter.keys(), ["thumbnail"])
+        no_assets = collection.get_assets(media_type=pystac.MediaType.HDF)
+        self.assertEqual(no_assets, {})
+
+    def test_removing_optional_attributes(self) -> None:
+        path = TestCases.get_path("data-files/collections/with-assets.json")
+        with open(path, "r") as file:
+            data = json.load(file)
+        data["title"] = "dummy title"
+        data["stac_extensions"] = ["dummy extension"]
+        data["keywords"] = ["key", "word"]
+        data["providers"] = [{"name": "pystac"}]
+        collection = pystac.Collection.from_dict(data)
+
+        # Assert we have everything set
+        assert collection.title
+        assert collection.stac_extensions
+        assert collection.keywords
+        assert collection.providers
+        assert collection.summaries
+        assert collection.assets
+
+        # Remove all of the optional stuff
+        collection.title = None
+        collection.stac_extensions = []
+        collection.keywords = []
+        collection.providers = []
+        collection.summaries = pystac.Summaries({})
+        collection.assets = {}
+
+        collection_as_dict = collection.to_dict()
+        for key in (
+            "title",
+            "stac_extensions",
+            "keywords",
+            "providers",
+            "summaries",
+            "assets",
+        ):
+            assert key not in collection_as_dict
+
     def test_from_dict_preserves_dict(self) -> None:
         path = TestCases.get_path("data-files/collections/with-assets.json")
         with open(path) as f:
@@ -258,11 +332,48 @@ class CollectionTest(unittest.TestCase):
         with self.assertRaises(pystac.STACTypeError):
             _ = pystac.Collection.from_dict(catalog_dict)
 
+    def test_clone_preserves_assets(self) -> None:
+        path = TestCases.get_path("data-files/collections/with-assets.json")
+        original_collection = Collection.from_file(path)
+        assert len(original_collection.assets) > 0
+        assert all(
+            asset.owner is original_collection
+            for asset in original_collection.assets.values()
+        )
+
+        cloned_collection = original_collection.clone()
+
+        for key in original_collection.assets:
+            with self.subTest(f"Preserves {key} asset"):
+                self.assertIn(key, cloned_collection.assets)
+            cloned_asset = cloned_collection.assets.get(key)
+            if cloned_asset is not None:
+                with self.subTest(f"Sets owner for {key}"):
+                    self.assertIs(cloned_asset.owner, cloned_collection)
+
+    def test_to_dict_no_self_href(self) -> None:
+        temporal_extent = TemporalExtent(intervals=[[TEST_DATETIME, None]])
+        spatial_extent = SpatialExtent(bboxes=ARBITRARY_BBOX)
+        extent = Extent(spatial=spatial_extent, temporal=temporal_extent)
+        collection = Collection(
+            id="an-id", description="A test Collection", extent=extent
+        )
+        d = collection.to_dict(include_self_link=False)
+        Collection.from_dict(d)
+
 
 class ExtentTest(unittest.TestCase):
     def setUp(self) -> None:
         self.maxDiff = None
 
+    def test_temporal_extent_init_typing(self) -> None:
+        # This test exists purely to test the typing of the intervals argument to
+        # TemporalExtent
+        start_datetime = str_to_datetime("2022-01-01T00:00:00Z")
+        end_datetime = str_to_datetime("2022-01-31T23:59:59Z")
+
+        _ = TemporalExtent([[start_datetime, end_datetime]])
+
     def test_spatial_allows_single_bbox(self) -> None:
         temporal_extent = TemporalExtent(intervals=[[TEST_DATETIME, None]])
 
@@ -402,3 +513,31 @@ class CollectionSubClassTest(unittest.TestCase):
         cloned_collection = custom_collection.clone()
 
         self.assertIsInstance(cloned_collection, self.BasicCustomCollection)
+
+
+def test_custom_collection_from_dict(collection: Collection) -> None:
+    # https://github.com/stac-utils/pystac/issues/862
+    class CustomCollection(Collection):
+        @classmethod
+        def from_dict(
+            cls,
+            d: Dict[str, Any],
+            href: Optional[str] = None,
+            root: Optional[Catalog] = None,
+            migrate: bool = False,
+            preserve_dict: bool = True,
+        ) -> CustomCollection:
+            return super().from_dict(d)
+
+    _ = CustomCollection.from_dict(collection.to_dict())
+
+
+@pytest.mark.parametrize("add_canonical", (True, False))
+def test_remove_hierarchical_links(
+    test_case_1_catalog: Catalog, add_canonical: bool
+) -> None:
+    collection = list(test_case_1_catalog.get_all_collections())[0]
+    collection.remove_hierarchical_links(add_canonical=add_canonical)
+    for link in collection.links:
+        assert not link.is_hierarchical()
+    assert bool(collection.get_single_link("canonical")) == add_canonical
