@@ -1,15 +1,20 @@
-from copy import deepcopy
-import os
+from __future__ import annotations
+
 import json
+import os
 import tempfile
-from typing import Any, Dict
 import unittest
+from copy import deepcopy
+from typing import Any, Dict, Optional
+
+import dateutil.relativedelta
+import pytest
 
 import pystac
-from pystac import Asset, Item
-from pystac.validation import validate_dict
 import pystac.serialization.common_properties
-from pystac.utils import datetime_to_str, get_opt, str_to_datetime, is_absolute_href
+from pystac import Asset, Catalog, Item
+from pystac.utils import datetime_to_str, get_opt, is_absolute_href, str_to_datetime
+from pystac.validation import validate_dict
 from tests.utils import TestCases, assert_to_from_dict
 
 
@@ -40,10 +45,10 @@ class ItemTest(unittest.TestCase):
         # test that the parameter is preserved
         self.assertEqual(param_dict, item_dict)
 
-        # assert that the parameter is not preserved with
-        # non-default parameter
+        # assert that the parameter is preserved regardless of
+        # preserve_dict
         _ = Item.from_dict(param_dict, preserve_dict=False)
-        self.assertNotEqual(param_dict, item_dict)
+        self.assertEqual(param_dict, item_dict)
 
     def test_from_dict_set_root(self) -> None:
         item_dict = self.get_example_item_dict()
@@ -52,7 +57,7 @@ class ItemTest(unittest.TestCase):
         self.assertIs(item.get_root(), catalog)
 
     def test_set_self_href_does_not_break_asset_hrefs(self) -> None:
-        cat = TestCases.test_case_2()
+        cat = TestCases.case_2()
         for item in cat.get_all_items():
             for asset in item.assets.values():
                 if is_absolute_href(asset.href):
@@ -62,7 +67,7 @@ class ItemTest(unittest.TestCase):
                 self.assertTrue(is_absolute_href(asset.href))
 
     def test_set_self_href_none_ignores_relative_asset_hrefs(self) -> None:
-        cat = TestCases.test_case_2()
+        cat = TestCases.case_2()
         for item in cat.get_all_items():
             for asset in item.assets.values():
                 if is_absolute_href(asset.href):
@@ -72,14 +77,28 @@ class ItemTest(unittest.TestCase):
                 self.assertFalse(is_absolute_href(asset.href))
 
     def test_asset_absolute_href(self) -> None:
+        item_path = TestCases.get_path("data-files/item/sample-item.json")
         item_dict = self.get_example_item_dict()
         item = Item.from_dict(item_dict)
+        item.set_self_href(item_path)
         rel_asset = Asset("./data.geojson")
         rel_asset.set_owner(item)
-        expected_href = os.path.abspath("./data.geojson")
+        expected_href = os.path.abspath(
+            os.path.join(os.path.dirname(item_path), "./data.geojson")
+        )
         actual_href = rel_asset.get_absolute_href()
         self.assertEqual(expected_href, actual_href)
 
+    def test_asset_absolute_href_no_item_self(self) -> None:
+        item_dict = self.get_example_item_dict()
+        item = Item.from_dict(item_dict)
+        assert item.get_self_href() is None
+
+        rel_asset = Asset("./data.geojson")
+        rel_asset.set_owner(item)
+        actual_href = rel_asset.get_absolute_href()
+        self.assertEqual(None, actual_href)
+
     def test_extra_fields(self) -> None:
         item = pystac.Item.from_file(
             TestCases.get_path("data-files/item/sample-item.json")
@@ -100,7 +119,7 @@ class ItemTest(unittest.TestCase):
             self.assertEqual(read_item.extra_fields["test"], "extra")
 
     def test_clearing_collection(self) -> None:
-        collection = TestCases.test_case_4().get_child("acc")
+        collection = TestCases.case_4().get_child("acc")
         assert isinstance(collection, pystac.Collection)
         item = next(iter(collection.get_all_items()))
         self.assertEqual(item.collection_id, collection.id)
@@ -147,6 +166,59 @@ class ItemTest(unittest.TestCase):
 
         null_dt_item.validate()
 
+    def test_get_assets(self) -> None:
+        item = pystac.Item.from_file(
+            TestCases.get_path("data-files/item/sample-item.json")
+        )
+
+        media_type_filter = item.get_assets(media_type=pystac.MediaType.COG)
+        self.assertCountEqual(media_type_filter.keys(), ["analytic"])
+        role_filter = item.get_assets(role="data")
+        self.assertCountEqual(role_filter.keys(), ["analytic"])
+        multi_filter = item.get_assets(
+            media_type=pystac.MediaType.PNG, role="thumbnail"
+        )
+        self.assertCountEqual(multi_filter.keys(), ["thumbnail"])
+
+        no_filter = item.get_assets()
+        self.assertCountEqual(no_filter.keys(), ["analytic", "thumbnail"])
+        no_assets = item.get_assets(media_type=pystac.MediaType.HDF)
+        self.assertEqual(no_assets, {})
+
+    def test_null_datetime_constructor(self) -> None:
+        item = pystac.Item.from_file(
+            TestCases.get_path("data-files/item/sample-item.json")
+        )
+        with self.assertRaises(pystac.STACError):
+            Item(
+                "test",
+                geometry=item.geometry,
+                bbox=item.bbox,
+                datetime=None,
+                end_datetime=item.datetime,
+                properties={},
+            )
+        with self.assertRaises(pystac.STACError):
+            Item(
+                "test",
+                geometry=item.geometry,
+                bbox=item.bbox,
+                datetime=None,
+                start_datetime=item.datetime,
+                properties={},
+            )
+        assert item.datetime
+        null_dt_item = Item(
+            "test",
+            geometry=item.geometry,
+            bbox=item.bbox,
+            datetime=None,
+            start_datetime=item.datetime,
+            end_datetime=item.datetime + dateutil.relativedelta.relativedelta(days=1),
+            properties={},
+        )
+        null_dt_item.validate()
+
     def test_get_set_asset_datetime(self) -> None:
         item = pystac.Item.from_file(
             TestCases.get_path("data-files/item/sample-item-asset-properties.json")
@@ -173,7 +245,7 @@ class ItemTest(unittest.TestCase):
         )
 
     def test_read_eo_item_owns_asset(self) -> None:
-        item = next(iter(TestCases.test_case_1().get_all_items()))
+        item = next(iter(TestCases.case_1().get_all_items()))
         assert len(item.assets) > 0
         for asset_key in item.assets:
             self.assertEqual(item.assets[asset_key].owner, item)
@@ -207,18 +279,26 @@ class ItemTest(unittest.TestCase):
         )
         self.assertFalse(did_merge)
 
-    def test_clone_sets_asset_owner(self) -> None:
-        cat = TestCases.test_case_2()
-        item = next(iter(cat.get_all_items()))
-        original_asset = list(item.assets.values())[0]
-        assert original_asset.owner is item
+    def test_clone_preserves_assets(self) -> None:
+        cat = TestCases.case_2()
+        original_item = next(iter(cat.get_all_items()))
+        assert len(original_item.assets) > 0
+        assert all(
+            asset.owner is original_item for asset in original_item.assets.values()
+        )
 
-        clone = item.clone()
-        clone_asset = list(clone.assets.values())[0]
-        self.assertIs(clone_asset.owner, clone)
+        cloned_item = original_item.clone()
+
+        for key in original_item.assets:
+            with self.subTest(f"Preserves {key} asset"):
+                self.assertIn(key, cloned_item.assets)
+            cloned_asset = cloned_item.assets.get(key)
+            if cloned_asset is not None:
+                with self.subTest(f"Sets owner for {key}"):
+                    self.assertIs(cloned_asset.owner, cloned_item)
 
     def test_make_asset_href_relative_is_noop_on_relative_hrefs(self) -> None:
-        cat = TestCases.test_case_2()
+        cat = TestCases.case_2()
         item = next(iter(cat.get_all_items()))
         asset = list(item.assets.values())[0]
         assert not is_absolute_href(asset.href)
@@ -235,6 +315,20 @@ class ItemTest(unittest.TestCase):
         with self.assertRaises(pystac.STACTypeError):
             _ = pystac.Item.from_dict(catalog_dict)
 
+    def test_geo_interface(self) -> None:
+        item = pystac.Item.from_file(
+            TestCases.get_path("data-files/item/sample-item.json")
+        )
+        self.assertEqual(item.geometry, item.__geo_interface__)
+
+    def test_relative_extension_path(self) -> None:
+        item = pystac.Item.from_file(
+            TestCases.get_path(
+                "data-files/item/sample-item-with-relative-extension-path.json"
+            )
+        )
+        item.validate()
+
 
 class ItemSubClassTest(unittest.TestCase):
     """This tests cases related to creating classes inheriting from pystac.Catalog to
@@ -266,6 +360,44 @@ class ItemSubClassTest(unittest.TestCase):
         self.assertIsInstance(cloned_item, self.BasicCustomItem)
 
 
+class AssetTest(unittest.TestCase):
+    def setUp(self) -> None:
+        self.maxDiff = None
+        with open(TestCases.get_path("data-files/item/sample-item.json")) as src:
+            item_dict = json.load(src)
+
+        self.asset_dict = item_dict["assets"]["analytic"]
+
+    def example_asset(self) -> Asset:
+        return Asset.from_dict(self.asset_dict)
+
+    def test_clone(self) -> None:
+        original_asset = self.example_asset()
+        cloned_asset = original_asset.clone()
+
+        self.assertDictEqual(original_asset.to_dict(), self.asset_dict)
+        self.assertDictEqual(cloned_asset.to_dict(), self.asset_dict)
+
+        # Changes to original asset should not affect cloned Asset
+        original_asset.description = "Some new description"
+        self.assertDictEqual(cloned_asset.to_dict(), self.asset_dict)
+
+        original_asset.href = "/path/to/new/href"
+        self.assertDictEqual(cloned_asset.to_dict(), self.asset_dict)
+
+        original_asset.title = "New Title"
+        self.assertDictEqual(cloned_asset.to_dict(), self.asset_dict)
+
+        original_asset.roles = ["new role"]
+        self.assertDictEqual(cloned_asset.to_dict(), self.asset_dict)
+
+        original_asset.roles.append("new role")
+        self.assertDictEqual(cloned_asset.to_dict(), self.asset_dict)
+
+        original_asset.extra_fields["new_field"] = "new_value"
+        self.assertDictEqual(cloned_asset.to_dict(), self.asset_dict)
+
+
 class AssetSubClassTest(unittest.TestCase):
     class CustomAsset(Asset):
         pass
@@ -287,3 +419,50 @@ class AssetSubClassTest(unittest.TestCase):
         cloned_asset = asset.clone()
 
         self.assertIsInstance(cloned_asset, self.CustomAsset)
+        self.assertIsInstance(cloned_asset, self.CustomAsset)
+
+
+def test_custom_item_from_dict(item: Item) -> None:
+    # https://github.com/stac-utils/pystac/issues/862
+    class CustomItem(Item):
+        @classmethod
+        def from_dict(
+            cls,
+            d: Dict[str, Any],
+            href: Optional[str] = None,
+            root: Optional[Catalog] = None,
+            migrate: bool = False,
+            preserve_dict: bool = True,
+        ) -> CustomItem:
+            return super().from_dict(d)
+
+    _ = CustomItem.from_dict(item.to_dict())
+
+
+def test_item_from_dict_raises_useful_error() -> None:
+    item_dict = {"type": "Feature", "stac_version": "1.0.0", "id": "lalalalala"}
+    with pytest.raises(pystac.STACError, match="Invalid Item: "):
+        Item.from_dict(item_dict)
+
+
+def test_item_from_dict_with_missing_stac_version_raises_useful_error() -> None:
+    item_dict = {"type": "Feature", "id": "lalalalala"}
+    with pytest.raises(pystac.STACTypeError, match="'stac_version' is missing"):
+        Item.from_dict(item_dict)
+
+
+def test_item_from_dict_with_missing_type_raises_useful_error() -> None:
+    item_dict = {"stac_version": "0.8.0", "id": "lalalalala"}
+    with pytest.raises(pystac.STACTypeError, match="'type' is missing"):
+        Item.from_dict(item_dict)
+
+
+@pytest.mark.parametrize("add_canonical", (True, False))
+def test_remove_hierarchical_links(
+    test_case_1_catalog: Catalog, add_canonical: bool
+) -> None:
+    item = list(test_case_1_catalog.get_all_items())[0]
+    item.remove_hierarchical_links(add_canonical=add_canonical)
+    for link in item.links:
+        assert not link.is_hierarchical()
+    assert bool(item.get_single_link("canonical")) == add_canonical
