@@ -1,13 +1,19 @@
-import datetime
+import json
 import os
 import unittest
+from datetime import datetime
+from pathlib import Path
 from tempfile import TemporaryDirectory
 from typing import Any, Dict, List
 
+import pytest
+
 import pystac
+from pystac import Collection, Item, Link
+from pystac.link import HIERARCHICAL_LINKS
 from tests.utils.test_cases import ARBITRARY_EXTENT
 
-TEST_DATETIME: datetime.datetime = datetime.datetime(2020, 3, 14, 16, 32)
+TEST_DATETIME: datetime = datetime(2020, 3, 14, 16, 32)
 
 
 class LinkTest(unittest.TestCase):
@@ -22,6 +28,13 @@ class LinkTest(unittest.TestCase):
             properties={},
         )
 
+    def test_path_like(self) -> None:
+        rel = "some-rel"
+        target = os.path.abspath("../elsewhere")
+        link = pystac.Link(rel, target)
+
+        self.assertEqual(os.fspath(link), target)
+
     def test_minimal(self) -> None:
         rel = "my rel"
         target = "https://example.com/a/b"
@@ -130,6 +143,73 @@ class LinkTest(unittest.TestCase):
             finally:
                 os.chdir(previous)
 
+    def test_auto_title_when_resolved(self) -> None:
+        extent = pystac.Extent.from_items([self.item])
+        collection = pystac.Collection(
+            id="my_collection",
+            description="Test Collection",
+            extent=extent,
+            title="Collection Title",
+        )
+        link = pystac.Link("my rel", target=collection)
+
+        self.assertEqual(collection.title, link.title)
+
+    def test_auto_title_not_found(self) -> None:
+        extent = pystac.Extent.from_items([self.item])
+        collection = pystac.Collection(
+            id="my_collection",
+            description="Test Collection",
+            extent=extent,
+        )
+        link = pystac.Link("my rel", target=collection)
+
+        self.assertEqual(None, link.title)
+
+    def test_auto_title_is_serialized(self) -> None:
+        extent = pystac.Extent.from_items([self.item])
+        collection = pystac.Collection(
+            id="my_collection",
+            description="Test Collection",
+            extent=extent,
+            title="Collection Title",
+        )
+        link = pystac.Link("my rel", target=collection)
+
+        assert link.to_dict().get("title") == collection.title
+
+    def test_no_auto_title_if_not_resolved(self) -> None:
+        link = pystac.Link(
+            "my rel", target="https://www.some-domain.com/path/to/thing.txt"
+        )
+
+        assert link.title is None
+
+    def test_title_as_init_argument(self) -> None:
+        link_title = "Link title"
+        extent = pystac.Extent.from_items([self.item])
+        collection = pystac.Collection(
+            id="my_collection",
+            description="Test Collection",
+            extent=extent,
+            title="Collection Title",
+        )
+        link = pystac.Link("my rel", title=link_title, target=collection)
+
+        assert link.title == link_title
+        assert link.to_dict().get("title") == link_title
+
+    def test_serialize_link(self) -> None:
+        href = "https://some-domain/path/to/item.json"
+        title = "A Test Link"
+        link = pystac.Link(pystac.RelType.SELF, href, pystac.MediaType.JSON, title)
+        link_dict = link.to_dict()
+
+        self.assertEqual(str(link_dict["rel"]), "self")
+        self.assertEqual(str(link_dict["type"]), "application/json")
+        self.assertEqual(link_dict["title"], title)
+        self.assertEqual(link_dict["href"], href)
+
 
 class StaticLinkTest(unittest.TestCase):
     def setUp(self) -> None:
@@ -231,3 +311,37 @@ class LinkInheritanceTest(unittest.TestCase):
         )
         cloned_link = link.clone()
         self.assertIsInstance(cloned_link, self.CustomLink)
+
+
+def test_relative_self_link(tmp_path: Path) -> None:
+    # https://github.com/stac-utils/pystac/issues/801
+    item = Item("an-id", None, None, datetime.now(), {})
+    item_as_dict = item.to_dict(include_self_link=False)
+    item_as_dict["links"] = [{"href": "item.json", "rel": "self"}]
+    item_as_dict["assets"] = {"data": {"href": "asset.tif"}}
+    with open(tmp_path / "item.json", "w") as f:
+        json.dump(item_as_dict, f)
+    Path(tmp_path / "asset.tif").touch()
+    collection = Collection("an-id", "a description", ARBITRARY_EXTENT)
+    collection.add_link(Link("item", "item.json"))
+    collection.save_object(
+        include_self_link=False, dest_href=str(tmp_path / "collection.json")
+    )
+    collection = Collection.from_file(str(tmp_path / "collection.json"))
+    read_item = collection.get_item("an-id")
+    assert read_item
+    asset_href = read_item.assets["data"].get_absolute_href()
+    assert asset_href
+    assert Path(asset_href).exists()
+
+
+@pytest.mark.parametrize("rel", HIERARCHICAL_LINKS)
+def test_is_hierarchical(rel: str) -> None:
+    assert Link(rel, "a-target").is_hierarchical()
+
+
+@pytest.mark.parametrize(
+    "rel", ["canonical", "derived_from", "alternate", "via", "prev", "next", "preview"]
+)
+def test_is_not_hierarchical(rel: str) -> None:
+    assert not Link(rel, "a-target").is_hierarchical()
