@@ -1,9 +1,12 @@
+import json
 import os
-import unittest
 import tempfile
+import unittest
+
+import pytest
 
 import pystac
-from pystac.stac_io import StacIO, DefaultStacIO, DuplicateKeyReportingMixin
+from pystac.stac_io import DefaultStacIO, DuplicateKeyReportingMixin, StacIO
 from tests.utils import TestCases
 
 
@@ -98,3 +101,44 @@ class StacIOTest(unittest.TestCase):
                 str(excinfo.exception),
                 f'Found duplicate object name "key" in {src_href}',
             )
+
+    @unittest.mock.patch("pystac.stac_io.urlopen")
+    def test_headers_stac_io(self, urlopen_mock: unittest.mock.MagicMock) -> None:
+        stac_io = DefaultStacIO(headers={"Authorization": "api-key fake-api-key-value"})
+
+        catalog = pystac.Catalog("an-id", "a description").to_dict()
+        # required until https://github.com/stac-utils/pystac/pull/896 is merged
+        catalog["links"] = []
+        urlopen_mock.return_value.__enter__.return_value.read.return_value = json.dumps(
+            catalog
+        ).encode("utf-8")
+        pystac.Catalog.from_file("https://example.com/catalog.json", stac_io=stac_io)
+
+        request_obj = urlopen_mock.call_args[0][0]
+        self.assertEqual(request_obj.headers, stac_io.headers)
+
+
+@pytest.mark.vcr()
+def test_retry_stac_io() -> None:
+    # This doesn't test any retry behavior, but it does make sure that we can
+    # still read objects.
+    _ = pytest.importorskip("urllib3")
+    from pystac.stac_io import RetryStacIO
+
+    stac_io = RetryStacIO()
+    _ = stac_io.read_stac_object("https://planetarycomputer.microsoft.com/api/stac/v1")
+
+
+@pytest.mark.vcr()
+def test_retry_stac_io_404() -> None:
+    # This doesn't test any retry behavior, but it does make sure that we can
+    # error when an object doesn't exist.
+    _ = pytest.importorskip("urllib3")
+    from pystac.stac_io import RetryStacIO
+
+    stac_io = RetryStacIO()
+    with pytest.raises(Exception):
+        _ = stac_io.read_stac_object(
+            "https://planetarycomputer.microsoft.com"
+            "/api/stac/v1/collections/not-a-collection-id"
+        )
