@@ -1,16 +1,33 @@
-# flake8: noqa
-
-from tests.utils.test_cases import (TestCases, RANDOM_GEOM, RANDOM_BBOX, RANDOM_EXTENT)
-
+__all__ = [
+    "TestCases",
+    "ARBITRARY_GEOM",
+    "ARBITRARY_BBOX",
+    "ARBITRARY_EXTENT",
+    "MockStacIO",
+]
+import unittest
 from copy import deepcopy
 from datetime import datetime
+from typing import Any, Dict, Type
+
 from dateutil.parser import parse
 
+import pystac
 from tests.utils.stac_io_mock import MockStacIO
+from tests.utils.test_cases import (
+    ARBITRARY_BBOX,
+    ARBITRARY_EXTENT,
+    ARBITRARY_GEOM,
+    TestCases,
+)
 
 
-def test_to_from_dict(test_class, stac_object_class, d):
-    def _parse_times(a_dict):
+def assert_to_from_dict(
+    test_class: unittest.TestCase,
+    stac_object_class: Type[pystac.STACObject],
+    d: Dict[str, Any],
+) -> None:
+    def _parse_times(a_dict: Dict[str, Any]) -> None:
         for k, v in a_dict.items():
             if isinstance(v, dict):
                 _parse_times(v)
@@ -19,7 +36,7 @@ def test_to_from_dict(test_class, stac_object_class, d):
                     if isinstance(vv, dict):
                         _parse_times(vv)
             else:
-                if k == 'datetime':
+                if k == "datetime":
                     if not isinstance(v, datetime):
                         a_dict[k] = parse(v)
                         a_dict[k] = a_dict[k].replace(microsecond=0)
